(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    346562,      10691]
NotebookOptionsPosition[    298894,       9417]
NotebookOutlinePosition[    318430,       9836]
CellTagsIndexPosition[    317695,       9809]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Induction and Recursion", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 5}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 "In this chapter we describe how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to help you make conjectures and prove them with mathematical \
induction and strong induction. We will also look at several examples of \
using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to explore recursive definitions and to implement recursive algorithms. \
Recursion is an important tool in any computer programming language, and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is no exception. We conclude with an example of proving program \
correctness of a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function. "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.1 Mathematical Induction", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "In this section we will demonstrate how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " both to discover propositions and to aid in the use of mathematical \
induction to verify them. We begin with two examples of how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to discover and prove summation formulas. We then consider a question of \
divisibility. "
}], "Text"],

Cell[CellGroupData[{

Cell["Summation Example 1", "Subsection",
 CellTags->"Section1a"],

Cell["\<\
As our first example, we will explore a formula that you've already seen: \
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "1"}], "TI"], "n",
     LimitsPositioning->False], "i"}], "=", 
   RowBox[{
    RowBox[{"1", "+", "2", "+", "3", "+", "\[CenterEllipsis]", "+", "n"}], 
    "=", 
    FractionBox[
     RowBox[{"n", "(", 
      RowBox[{"n", "+", "1"}], ")"}], "2"]}]}], TraditionalForm]], "Equation"],

Cell["\<\
This formula is the subject of Example 1 in Section 5.1 of the text. Here, we \
will proceed as if we did not already know the formula. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Listing and Graphing to Find the Formula", "Subsubsection"],

Cell[TextData[{
 "Our first step is to discover the formula. To do this, we will have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " compute the sums for a variety of values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", using the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function. "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function has essentially the same syntax as ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". The first argument is an expression representing the values to be added \
in terms of a variable. The second argument is a list indicating the values \
of the variable over which the sum should be computed. Refer to Section 2.4 \
for a  description of the behavior of the second argument. A table \
summarizing the main possibilities is shown below."
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], "}"}]], "myCode"], Cell[TextData[{
     "sum from ",
     Cell[BoxData[
      FormBox[
       RowBox[{"i", "=", "1"}], TraditionalForm]]],
     " to ",
     Cell[BoxData[
      FormBox[
       SubscriptBox["i", "max"], TraditionalForm]]]
    }]]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "min"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], "}"}]], "myCode"], 
    RowBox[{"sum", " ", "from", " ", Cell[TextData[{
      Cell[BoxData[
       FormBox[
        RowBox[{"i", "=", 
         SubscriptBox["i", "min"]}], TraditionalForm]]],
      " to ",
      Cell[BoxData[
       FormBox[
        SubscriptBox["i", "max"], TraditionalForm]]]
     }]]}]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "min"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["i", "max"], "CodeVar"], ",", 
      StyleBox["step", "CodeVar"], "}"}]], "myCode"], 
    RowBox[{"sum", " ", "from", " ", Cell[TextData[{
      Cell[BoxData[
       FormBox[
        RowBox[{"i", "=", 
         SubscriptBox["i", "min"]}], TraditionalForm]]],
      " to ",
      Cell[BoxData[
       FormBox[
        SubscriptBox["i", "max"], TraditionalForm]]],
      " by ",
      Cell[BoxData[
       FormBox["step", TraditionalForm]]]
     }]]}]},
   {Cell[BoxData[
     RowBox[{"{", 
      StyleBox["i", "CodeVar"], ",", 
      StyleBox["list", "CodeVar"], "}"}]], "myCode"], 
    RowBox[{
    "sum", " ", "over", " ", Cell["i\[Element]list"], Cell[TextData[StyleBox[
     "", "Input"]]]}]}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]],
 FontFamily->"Times"]], "Text",
 TextAlignment->Center],

Cell["\<\
In our situation, we want to add the first several positive integers. For \
example, the sum of the first ten positive integers is \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{"i", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["55"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To discover the formula for the sum of the first ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " positive integers, we need several specific examples to analyze. To \
calculate a lot of examples at once, we will embed the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function inside of a ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ", with the maximum value of the summation as the table variable. This will \
produce the list of the sums of the first ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " positive integers for different values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{"i", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "50"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "3", ",", "6", ",", "10", ",", "15", ",", "21", ",", "28", ",", 
   "36", ",", "45", ",", "55", ",", "66", ",", "78", ",", "91", ",", "105", 
   ",", "120", ",", "136", ",", "153", ",", "171", ",", "190", ",", "210", 
   ",", "231", ",", "253", ",", "276", ",", "300", ",", "325", ",", "351", 
   ",", "378", ",", "406", ",", "435", ",", "465", ",", "496", ",", "528", 
   ",", "561", ",", "595", ",", "630", ",", "666", ",", "703", ",", "741", 
   ",", "780", ",", "820", ",", "861", ",", "903", ",", "946", ",", "990", 
   ",", "1035", ",", "1081", ",", "1128", ",", "1176", ",", "1225", ",", 
   "1275"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Remember that we're working as if we do not already know the answer. Just \
looking at the data, you might notice a pattern, but if not, it's sometimes \
helpful to pair the value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " with the result. To do this, we only need to modify the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function so that the first argument is the list whose first element is ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and whose second is the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"n", ",", 
     RowBox[{"Sum", "[", 
      RowBox[{"i", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "n"}], "}"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "50"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "10"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "15"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "21"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7", ",", "28"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8", ",", "36"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9", ",", "45"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10", ",", "55"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"11", ",", "66"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"12", ",", "78"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13", ",", "91"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"14", ",", "105"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"15", ",", "120"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16", ",", "136"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"17", ",", "153"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18", ",", "171"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19", ",", "190"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"20", ",", "210"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"21", ",", "231"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22", ",", "253"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"23", ",", "276"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24", ",", "300"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25", ",", "325"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"26", ",", "351"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27", ",", "378"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"28", ",", "406"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"29", ",", "435"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"30", ",", "465"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"31", ",", "496"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"32", ",", "528"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"33", ",", "561"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"34", ",", "595"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"35", ",", "630"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"36", ",", "666"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"37", ",", "703"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"38", ",", "741"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"39", ",", "780"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"40", ",", "820"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"41", ",", "861"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"42", ",", "903"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"43", ",", "946"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"44", ",", "990"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"45", ",", "1035"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"46", ",", "1081"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"47", ",", "1128"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"48", ",", "1176"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"49", ",", "1225"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"50", ",", "1275"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The entry ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"23", ",", "276"}], "}"}], TraditionalForm]]],
 " indicates that the sum of the first ",
 Cell[BoxData[
  FormBox["23", TraditionalForm]]],
 " positive integers is ",
 Cell[BoxData[
  FormBox["276", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "This still may not be enough to get an idea of what the formula could be, \
in which case a graph of the data can be of use. The ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " function is used to graph functions and data, and was first discussed in \
Section 2.3 of this manual. In this situation, we want to plot the points \
that the previous application of ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " produced: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"3", ",", "6"}], "}"}], ",", "\[Ellipsis]"}], 
   TraditionalForm]]],
 ". To do this, we apply ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " to the output above. Note that either of the previous two lists can be \
used. Given a list of pairs, ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " produces the graph interpreting the pairs as ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " coordinate pairs. Given a list of values, the values are interpreted as \
the ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " coordinates with the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " coordinate being the position of the value in the list. That is, the first \
element is plotted at ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-value 1, the second element at ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-value 2, etc. In this case, the plots are identical, so we will use the \
shorter of the two expressions. Since ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " computes the sums quickly, we'll go out to a maximum of n = 1000. That \
way, we'll be sure to have a good idea of the shape of the graph. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Sum", "[", 
     RowBox[{"i", ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "1000"}], "}"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, 
   {RGBColor[0.24720000000000014`, 0.24, 0.6], PointBox[CompressedData["
1:eJxd233YFGXZx/GRUFelGhGfliQcE3Uz1BFRV0MdFXVLkKmQNkIdFWwT0FFJ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     "]]}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 1000.}, {0, 500500.}},
  PlotRangeClipping->True,
  PlotRangePadding->{{20., 20.}, {10010., 10010.}}]], "Output"]
}, Open  ]],

Cell["\<\
The particular values in the list may not have been helpful at all in \
figuring out what kind of formula we were looking for. But this graph \
probably looks very familiar. It looks very much like the right half of a \
parabola, suggesting that the formula is quadratic. (Of course, it may be \
cubic or quartic or some other polynomial, but we'll start with the simplest \
possibility based on the graph.) \
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finding the Coefficients", "Subsubsection"],

Cell[TextData[{
 "Now that we have guessed the kind of formula, we can write it as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"a", "\[CenterDot]", 
      SuperscriptBox["n", "2"]}], "+", 
     RowBox[{"b", "\[CenterDot]", "n"}], "+", "c"}]}], TraditionalForm]]],
 ". Determining the coefficients ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " is our next task. We will have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " find them for us."
}], "Text"],

Cell["\<\
We already know a bunch of values for this function. Here are the first few \
again. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumList", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Sum", "[", 
     RowBox[{"i", ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "10"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "3", ",", "6", ",", "10", ",", "15", ",", "21", ",", "28", ",", 
   "36", ",", "45", ",", "55"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This data tells us a lot of information about our formula. For instance, if \
we plug in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "2"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "2", ")"}], "=", "3"}], TraditionalForm]]],
 ", meaning"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"3", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "\[CenterDot]", 
      SuperscriptBox["2", "2"]}], "+", 
     RowBox[{"b", "\[CenterDot]", "2"}], "+", "c"}], "=", 
    RowBox[{
     RowBox[{"4", "a"}], "+", 
     RowBox[{"2", "b"}], "+", "c"}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "For ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ", we have"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "=", 
   RowBox[{"a", "+", "b", "+", "c"}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "For ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "3"}], TraditionalForm]]],
 ":"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"6", "=", 
   RowBox[{
    RowBox[{"9", "a"}], "+", 
    RowBox[{"3", "b"}], "+", "c"}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can generate these equations for us. We can apply ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " with first argument an equation, formed with the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") operator, representing the function ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"a", "\[CenterDot]", 
      SuperscriptBox["n", "2"]}], "+", 
     RowBox[{"b", "\[CenterDot]", "n"}], "+", "c"}]}], TraditionalForm]]],
 ". Instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "n", ")"}], TraditionalForm]]],
 ", we use the data in ",
 StyleBox["sumList", "Input"],
 ". So our expression will be ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"sumList", "[", 
    RowBox[{"[", "n", "]"}], "]"}], "\[Equal]", "a", "*", "n", "^", "2", "+", 
   "b", "*", "n", "+", "c"}]], "myCode"],
 ".  "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumEquations", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sumList", "[", 
      RowBox[{"[", "n", "]"}], "]"}], "\[Equal]", 
     RowBox[{
      RowBox[{"a", "*", 
       RowBox[{"n", "^", "2"}]}], "+", 
      RowBox[{"b", "*", "n"}], "+", "c"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "10"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "\[Equal]", 
    RowBox[{"a", "+", "b", "+", "c"}]}], ",", 
   RowBox[{"3", "\[Equal]", 
    RowBox[{
     RowBox[{"4", " ", "a"}], "+", 
     RowBox[{"2", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"6", "\[Equal]", 
    RowBox[{
     RowBox[{"9", " ", "a"}], "+", 
     RowBox[{"3", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"10", "\[Equal]", 
    RowBox[{
     RowBox[{"16", " ", "a"}], "+", 
     RowBox[{"4", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"15", "\[Equal]", 
    RowBox[{
     RowBox[{"25", " ", "a"}], "+", 
     RowBox[{"5", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"21", "\[Equal]", 
    RowBox[{
     RowBox[{"36", " ", "a"}], "+", 
     RowBox[{"6", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"28", "\[Equal]", 
    RowBox[{
     RowBox[{"49", " ", "a"}], "+", 
     RowBox[{"7", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"36", "\[Equal]", 
    RowBox[{
     RowBox[{"64", " ", "a"}], "+", 
     RowBox[{"8", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"45", "\[Equal]", 
    RowBox[{
     RowBox[{"81", " ", "a"}], "+", 
     RowBox[{"9", " ", "b"}], "+", "c"}]}], ",", 
   RowBox[{"55", "\[Equal]", 
    RowBox[{
     RowBox[{"100", " ", "a"}], "+", 
     RowBox[{"10", " ", "b"}], "+", "c"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The list of equations stored as ",
 StyleBox["sumEquations", "Input"],
 " is a system of equations. In particular, we have 10 equations in three \
variables. You have probably seen systems of at least 2 and 3 equations in 2 \
or 3 variables in previous mathematics courses. You can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " solve the system of equations by applying the ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " function to the list. We provide ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " with the list of equations and the list of variables appearing in them. \
(The second argument can often be omitted and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will deduce the variables.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{"sumEquations", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", 
     FractionBox["1", "2"]}], ",", 
    RowBox[{"b", "\[Rule]", 
     FractionBox["1", "2"]}], ",", 
    RowBox[{"c", "\[Rule]", "0"}]}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You may recall that to solve a system of equations in three unknowns, only \
three equations are required. In this situation, having more equations is \
useful. If we were wrong about the formula being quadratic but attempted to \
find coefficients with only three equations, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " may still have found values for ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " that satisfied the three equations we chose. With ten equations, if the \
actual formula were not quadratic, there is a greater chance that no values \
of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " would satisfy all ten equations. In that case, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " would have returned an empty list to indicate the absence of a solution \
and that our guess about the kind of formula was incorrect. "
}], "Text"],

Cell[TextData[{
 "Let's review what we've done so far. Our goal is to find a formula for the \
sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "1"}], "TI"], "n",
     LimitsPositioning->False], "i"}], TraditionalForm]],
  CellChangeTimes->{{3.553527478053399*^9, 3.5535275179520683`*^9}}],
 ". We used ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to compute a bunch of values of this sum and graphed them. This graph \
suggested a quadratic formula, i.e., one of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", 
     SuperscriptBox["n", "2"]}], "+", 
    RowBox[{"b", "\[CenterDot]", "n"}], "+", "c"}], TraditionalForm]]],
 " for some values of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ". We then used ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " function to determine that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", 
    RowBox[{"b", "=", 
     FractionBox["1", "2"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "=", "0"}], TraditionalForm]]],
 ". In other words, we've found the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], 
     SuperscriptBox["n", "2"]}], "+", 
    RowBox[{
     FractionBox["1", "2"], "n"}]}], TraditionalForm]]],
 ", which, of course, is the same as ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"n", "(", 
     RowBox[{"n", "+", "1"}], ")"}], "2"], TraditionalForm]]],
 ". Although we have found a formula, we have not yet proven anything, we've \
only made a conjecture."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Induction Proof", "Subsubsection"],

Cell["\<\
To prove that our formula is correct, we use mathematical induction. First, \
let's make our formula into a function.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sumF", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "2"}], ")"}], "*", 
    RowBox[{"n", "^", "2"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "2"}], ")"}], "*", "n"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "To complete the basis step of the induction, we need to see that the \
formula agrees with the sum for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{"i", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumF", "[", "1", "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell["They are equal and the basis step is verified. ", "Text"],

Cell[TextData[{
 "For the inductive step, we assume that the formula is correct for ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " and need to demonstrate that it is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "+", "1"}], TraditionalForm]]],
 ". In Example 1 in the textbook, this was done by starting with the sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "+", "2", "+", "\[CenterEllipsis]", "+", "k", "+", 
    RowBox[{"(", 
     RowBox[{"k", "+", "1"}], ")"}]}], TraditionalForm]]],
 " and applying the inductive hypothesis to the first ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " terms to obtain ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"k", "(", 
      RowBox[{"k", "+", "1"}], ")"}], "2"], "+", 
    RowBox[{"(", 
     RowBox[{"k", "+", "1"}], ")"}]}], TraditionalForm]]],
 ". Then algebra is used to turn that expression into the formula evaluated \
at ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "+", "1"}], TraditionalForm]]],
 ". With ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can just check whether the expressions are the same. "
}], "Text"],

Cell[TextData[{
 "The sum of the first ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "+", "1"}], TraditionalForm]]],
 " terms is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "+", "2", "+", "\[CenterEllipsis]", "+", "k", "+", 
     RowBox[{"(", 
      RowBox[{"k", "+", "1"}], ")"}]}], "=", 
    RowBox[{
     RowBox[{"f", "(", "k", ")"}], "+", 
     RowBox[{"(", 
      RowBox[{"k", "+", "1"}], ")"}]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sumF", "[", "k", "]"}], "+", 
  RowBox[{"(", 
   RowBox[{"k", "+", "1"}], ")"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"1", "+", 
  FractionBox[
   RowBox[{"3", " ", "k"}], "2"], "+", 
  FractionBox[
   SuperscriptBox["k", "2"], "2"]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "That used the inductive hypothesis. We need to see if this is the same as \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", 
    RowBox[{"k", "+", "1"}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumF", "[", 
  RowBox[{"k", "+", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"1", "+", "k"}], "2"], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", "k"}], ")"}], "2"]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To check whether ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", 
    RowBox[{"k", "+", "1"}], ")"}], TraditionalForm]]],
 " is the same as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "k", ")"}], "+", 
    RowBox[{"(", 
     RowBox[{"k", "+", "1"}], ")"}]}], TraditionalForm]]],
 ", we will apply the ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " function. The first argument to ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " will be the equation formed by identifying the two expressions. The second \
argument is the variable ",
 StyleBox["k", "Input"],
 ". ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " will apply algebraic rules to the expression to determine the values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " for which the equation holds. If there is more than one equation or more \
than one variable, you can use a list in either argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"sumF", "[", "k", "]"}], "+", 
     RowBox[{"(", 
      RowBox[{"k", "+", "1"}], ")"}]}], "\[Equal]", 
    RowBox[{"sumF", "[", 
     RowBox[{"k", "+", "1"}], "]"}]}], ",", "k"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The output ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " indicates that the equation is valid for all values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", in particular, it is true for the positive integers. This means that the \
inductive step is verified, and hence the formula is correct."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Summation Example 2", "Subsection",
 CellTags->"Section1b"],

Cell[TextData[{
 "As a second example of using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find and prove summation formulae, consider the sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "1"}], "TI"], "n",
     LimitsPositioning->False], 
    RowBox[{
     RowBox[{
      SuperscriptBox["i", "2"], "(", 
      RowBox[{"i", "+", "1"}], ")"}], "!"}]}], TraditionalForm]],
  CellChangeTimes->{{3.553527478053399*^9, 3.5535275179520683`*^9}}],
 ". For this example, we won't go through the process of computing values and \
graphing as we did above. That is a very valuable process, and we strongly \
recommend that you go through it yourself with a few examples. But ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes a function that will give us the result. "
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Using ",
 StyleBox["Sum", "Input"],
 " to Determine the Formula"
}], "Subsubsection"],

Cell[TextData[{
 "Above, we used the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function to determine the numeric sum of a sequence of numbers. While the \
numbers being added were determined by a formula, nevertheless the bounds of \
the sum were explicit so that specific values could be calculated and added. \
To calculate ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "1"}], "TI"], "10",
     LimitsPositioning->False], 
    RowBox[{
     RowBox[{
      SuperscriptBox["i", "2"], "(", 
      RowBox[{"i", "+", "1"}], ")"}], "!"}]}], TraditionalForm]],
  CellChangeTimes->{{3.553527478053399*^9, 3.5535275179520683`*^9}}],
 ", we use ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " as in the previous example, with the formula given as the first argument. \
The bounds, ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " ranging from 1 to 10, are given in the second argument in the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"i", ",", "10"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"i", "^", "2"}], "*", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"i", "+", "1"}], ")"}], "!"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["4311014402"], "Output"]
}, Open  ]],

Cell[TextData[{
 "(Note the use of the exclamation mark for the computation of factorial. You \
can also use the functional form ",
 StyleBox[ButtonBox["Factorial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Factorial"], "FunctionLink"],
 " if you prefer.) "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function can also be used for computing symbolic sums, as in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "1"}], "TI"], "n",
     LimitsPositioning->False], 
    RowBox[{
     RowBox[{
      SuperscriptBox["i", "2"], "(", 
      RowBox[{"i", "+", "1"}], ")"}], "!"}]}], TraditionalForm]],
  CellChangeTimes->{{3.553527478053399*^9, 3.5535275179520683`*^9}}],
 ". All we need to do is replace the upper bound of 10 with the symbol ",
 StyleBox["n", "Input"],
 ", provided that ",
 StyleBox["n", "Input"],
 " has not been assigned a value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"i", "^", "2"}], "*", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"i", "+", "1"}], ")"}], "!"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "n"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"2", "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"2", "+", "n"}], ")"}], "!"}], "+", 
  RowBox[{"n", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "+", "n"}], ")"}], "!"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["Simplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Simplify"], "FunctionLink"],
 " to the output, we obtain a slightly simpler formula."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"2", "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "n"}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "+", "n"}], ")"}], "!"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that you can use symbols in place of any of the bound specifications. \
You can also use the symbol ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 " in place of the upper bound, provided the series is convergent."
}], "Text"],

Cell[TextData[{
 "Similarly, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the ",
 StyleBox[ButtonBox["Product",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Product"], "FunctionLink"],
 " function for computing numeric and symbolic products. The syntax is the \
same."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Induction Proof", "Subsubsection"],

Cell[TextData[{
 "Now that the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function has determined the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      StyleBox[
       RowBox[{"i", "=", "1"}], "TI"], "n",
      LimitsPositioning->False], 
     RowBox[{
      RowBox[{
       SuperscriptBox["i", "2"], "(", 
       RowBox[{"i", "+", "1"}], ")"}], "!"}]}], " ", "=", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"n", "-", "1"}], ")"}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", "+", "2"}], ")"}], "!"}]}], "+", "2"}]}], 
   TraditionalForm]],
  CellChangeTimes->{{3.553604401518861*^9, 3.5536044232207212`*^9}}],
 ", we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to help us prove it. Even though we can be very confident that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has given us a correct formula, applying a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function is not the same as a proof."
}], "Text"],

Cell["As before, we'll define a function based on the formula.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sumF2", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"n", "-", "1"}], ")"}], "*", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", "+", "2"}], ")"}], "!"}]}], "+", "2"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "For the basis step of the induction, we need to see that the formula holds \
for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ". The value of the sum for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 " is"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "^", "2"}], "*", 
  RowBox[{"(", 
   RowBox[{"1", "+", "1"}], ")"}]}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell[TextData[{
 "And the formula applied to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 " is"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumF2", "[", "1", "]"}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell["The basis step is verified.", "Text"],

Cell[TextData[{
 "For the inductive step, we assume that the formula is correct for ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". That is, we assume that"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "1"}], "TI"], "k",
     LimitsPositioning->False], 
    RowBox[{
     RowBox[{
      SuperscriptBox["i", "2"], "(", 
      RowBox[{"i", "+", "1"}], ")"}], "!"}]}], " ", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"k", "-", "1"}], ")"}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"k", "+", "2"}], ")"}], "!"}]}], "+", "2"}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "We need to show that the formula works for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "+", "1"}], TraditionalForm]]],
 ". Now,"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "1"}], "TI"], 
     RowBox[{"k", "+", "1"}],
     LimitsPositioning->False], 
    RowBox[{
     RowBox[{
      SuperscriptBox["i", "2"], "(", 
      RowBox[{"i", "+", "1"}], ")"}], "!"}]}], " ", "=", 
   RowBox[{
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      StyleBox[
       RowBox[{"i", "=", "1"}], "TI"], "k",
      LimitsPositioning->False], 
     RowBox[{
      RowBox[{
       SuperscriptBox["i", "2"], "(", 
       RowBox[{"i", "+", "1"}], ")"}], "!"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "+", "1"}], ")"}], "2"], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"k", "+", "1"}], ")"}], "+", "1"}], ")"}], "!"}]}]}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "Using the inductive hypothesis that the formula is correct for ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", this is"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sumF2", "[", "k", "]"}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "+", "1"}], ")"}], "^", "2"}], "*", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"k", "+", "1"}], ")"}], "+", "1"}], ")"}], "!"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"2", "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "k"}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "+", "k"}], ")"}], "!"}]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", "k"}], ")"}], "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "+", "k"}], ")"}], "!"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We need to check to see if this is the same as the formula applied to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "+", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumF2", "[", 
  RowBox[{"k", "+", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"2", "+", 
  RowBox[{"k", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"3", "+", "k"}], ")"}], "!"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Again, we use ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " applied to the equation formed by identifying these two expressions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sumF2", "[", "k", "]"}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"k", "+", "1"}], ")"}], "^", "2"}], "*", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"k", "+", "1"}], ")"}], "+", "1"}], ")"}], "!"}]}]}], 
   "\[Equal]", 
   RowBox[{"sumF2", "[", 
    RowBox[{"k", "+", "1"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Reduce", "::", "fexp"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Warning: \[NoBreak]\\!\\(Reduce\\)\[NoBreak] used \
FunctionExpand to transform the system. Since FunctionExpand transformation \
rules are only generically correct, the solution set might have been altered. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/Reduce\\\", ButtonNote -> \
\\\"Reduce::fexp\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that a warning message was produced this time. The reason for this \
is the inclusion of the factorials, which ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " treats with particular care. However, the output is correct, the equation \
is true, as you can verify by hand."
}], "Text"],

Cell[TextData[{
 "It should be pointed out that this \[OpenCurlyDoubleQuote]proof\
\[CloseCurlyDoubleQuote] of the formula for the sum is only correct modulo \
the correctness of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions that we have used and the proper functioning of the computer it \
is run on. One of the drawbacks of computer-aided proof is the degree to \
which your computer or software could be introducing hidden errors."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Divisibility", "Subsection",
 CellTags->"Section1c"],

Cell[TextData[{
 "For a final example, we'll see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to help prove results about divisibility. In Example 8 from \
Section 5.1 of the text, it was shown that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "3"], "-", "n"}], TraditionalForm]]],
 " is divisible by 3 for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Exercise 33 asks you to prove that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "5"], "-", "n"}], TraditionalForm]]],
 " is divisible by 5 for all positive integers. These two facts suggest that \
perhaps ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "7"], "-", "n"}], TraditionalForm]]],
 " is divisible by 7 for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Let's see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can help us prove this fact. "
}], "Text"],

Cell[TextData[{
 "We begin by creating a function to represent the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "7"], "-", "n"}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"divisibleEx", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"n", "^", "7"}], "-", "n"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The basis case is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ". For ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ", the expression is"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"divisibleEx", "[", "1", "]"}]], "Input"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell["This is divisible by 7, so the basis case holds.", "Text"],

Cell[TextData[{
 "The inductive hypothesis is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["k", "7"], "-", "k"}], TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox["7", TraditionalForm]]],
 " for a positive integer ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". We need to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "+", "1"}], ")"}], "7"], "-", 
    RowBox[{"(", 
     RowBox[{"k", "+", "1"}], ")"}]}], TraditionalForm]]],
 " is divisible by 7. To do this, we'll use the following fact: if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "|", "a"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "|", 
    RowBox[{"b", "-", "a"}]}], TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "|", "b"}], TraditionalForm]]],
 ". (The reader should verify this statement, which is equivalent to Theorem \
1, part (i), of Section 4.1.) "
}], "Text"],

Cell[TextData[{
 "By assumption, 7 divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["k", "7"], "-", "k"}], TraditionalForm]]],
 ". We will have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " compute the difference with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "+", "1"}], ")"}], "7"], "-", 
    RowBox[{"(", 
     RowBox[{"k", "+", "1"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"divisibleEx", "[", 
   RowBox[{"k", "+", "1"}], "]"}], "-", 
  RowBox[{"divisibleEx", "[", "k", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "1"}], "-", 
  SuperscriptBox["k", "7"], "+", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "+", "k"}], ")"}], "7"]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We apply the ",
 StyleBox[ButtonBox["Expand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Expand"], "FunctionLink"],
 " function to the result."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand", "[", "%", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"7", " ", "k"}], "+", 
  RowBox[{"21", " ", 
   SuperscriptBox["k", "2"]}], "+", 
  RowBox[{"35", " ", 
   SuperscriptBox["k", "3"]}], "+", 
  RowBox[{"35", " ", 
   SuperscriptBox["k", "4"]}], "+", 
  RowBox[{"21", " ", 
   SuperscriptBox["k", "5"]}], "+", 
  RowBox[{"7", " ", 
   SuperscriptBox["k", "6"]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can see that the coefficients are all multiples of 7 and thus the \
expression is divisible by 7. We can confirm this with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by checking that the difference is divisible by 7 by using the ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " function on the equation formed by setting the difference equal to 0 and \
invoking the ",
 StyleBox[ButtonBox["Modulus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Modulus"], "FunctionLink"],
 " option."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"divisibleEx", "[", 
      RowBox[{"k", "+", "1"}], "]"}], "-", 
     RowBox[{"divisibleEx", "[", "k", "]"}]}], "\[Equal]", "0"}], ",", 
   RowBox[{"Modulus", "->", "7"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Thus, the inductive step is verified and hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "7"], "-", "n"}], TraditionalForm]]],
 " is divisible by 7 for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["5.2 Strong Induction and Well-Ordering", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 "In this section, we'll see one way that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to support a proof by strong induction. In particular, we will \
consider the class of problems illustrated in Example 4: prove that every \
amount of postage of 12 cents or more can be formed using just 4-cent and \
5-cent stamps. The second solution to that example will form the model for \
this discussion. (See also Exercises 3 through 8 as other examples of \
problems of this kind.) "
}], "Text"],

Cell[TextData[{
 "The basis step of the induction argument requires several propositions to \
be verified. Using the notation in the text, the propositions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"P", "(", "b", ")"}], ",", 
    RowBox[{"P", "(", 
     RowBox[{"b", "+", "1"}], ")"}], ",", "\[Ellipsis]", ",", 
    RowBox[{"P", "(", 
     RowBox[{"b", "+", "j"}], ")"}]}], TraditionalForm]]],
 " must all be demonstrated for some integer ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " and a positive integer ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ". ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be useful in these situations because it can often verify the basis \
cases for you. This is particularly useful when ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " is large."
}], "Text"],

Cell[TextData[{
 "Showing that every amount of postage over 12 cents can be formed using 4 \
and 5 cent stamps requires 4 basis cases: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"P", "(", "12", ")"}], ",", 
    RowBox[{"P", "(", "13", ")"}], ",", 
    RowBox[{"P", "(", "14", ")"}], ",", 
    RowBox[{"P", "(", "15", ")"}]}], TraditionalForm]]],
 ". We begin by showing how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to demonstrate ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "12", ")"}], TraditionalForm]]],
 ", that postage of 12 cents can be formed using 4 and 5 cent stamps. While \
you may object that it is obvious that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"12", "=", 
    RowBox[{"4", "\[CenterDot]", "3"}]}], TraditionalForm]]],
 ", our ultimate goal is to generalize our code to encompass the entire class \
of postage problems."
}], "Text"],

Cell[CellGroupData[{

Cell["Making Postage", "Subsubsection"],

Cell[TextData[{
 "To verify ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "12", ")"}], TraditionalForm]]],
 ", we must find nonnegative integers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", representing the number of stamps of the two denominations, such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"12", "=", 
    RowBox[{
     RowBox[{"4", "a"}], "+", 
     RowBox[{"5", "b"}]}]}], TraditionalForm]]],
 ". The most straightforward way of finding ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is to test all the possibilities. We know that ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " must both be nonnegative. "
}], "Text"],

Cell[TextData[{
 "Also, the maximum possible values of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftFloor]", 
    FractionBox["12", "4"], "\[RightFloor]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftFloor]", 
    FractionBox["12", "5"], "\[RightFloor]"}], TraditionalForm]]],
 ", respectively. (Recall that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftFloor]", "x", "\[RightFloor]"}], TraditionalForm]]],
 " is the notation used to represent the floor of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", that is, the largest integer less than or equal to ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".) To see why this is so, consider ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ": ",
 Cell[BoxData[
  FormBox[
   RowBox[{"12", "=", 
    RowBox[{
     RowBox[{
      RowBox[{"4", "a"}], "+", 
      RowBox[{"5", "b"}]}], "\[GreaterEqual]", 
     RowBox[{"5", "b"}]}]}], TraditionalForm]]],
 " since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[GreaterEqual]", "0"}], TraditionalForm]]],
 ". That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"5", "b"}], "\[LessEqual]", "12"}], TraditionalForm]]],
 " and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[LessEqual]", 
    FractionBox["12", "5"]}], TraditionalForm]]],
 ". And since ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " must be an integer, we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[LessEqual]", 
    RowBox[{"\[LeftFloor]", 
     FractionBox["12", "5"], "\[RightFloor]"}]}], TraditionalForm]]],
 ". Likewise for ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " must be between 0 and the floor of 12 divided by 4 or 5, respectively, we \
can use a pair of nested ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loops to check all the possible values. Within the loops, we only need to \
check to see if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"4", "a"}], "+", 
     RowBox[{"5", "b"}]}], "=", "12"}], TraditionalForm]]],
 ". If so, we'll terminate the loops, which we enclose in a ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 ", by applying ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " to the pair ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Catch", "[", "\[IndentingNewLine]", 
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"a", "=", "0"}], ",", 
    RowBox[{"a", "\[LessEqual]", 
     RowBox[{"Floor", "[", 
      RowBox[{"12", "/", "4"}], "]"}]}], ",", 
    RowBox[{"a", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{"For", "[", 
     RowBox[{
      RowBox[{"b", "=", "0"}], ",", 
      RowBox[{"b", "\[LessEqual]", 
       RowBox[{"Floor", "[", 
        RowBox[{"12", "/", "5"}], "]"}]}], ",", 
      RowBox[{"b", "++"}], ",", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"4", "*", "a"}], "+", 
          RowBox[{"5", "*", "b"}]}], "\[Equal]", "12"}], ",", 
        RowBox[{"Throw", "[", 
         RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}], "]"}]}], "]"}]}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], 
  "\[IndentingNewLine]", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can very easily generalize this by replacing the target value and the \
stamp denominations with variables. The following function implements this \
generalization. It accepts the two denominations and the target value as \
input and returns a list whose elements represent the number of each kind of \
stamp required. In the event that the function fails to form the desired \
amount of postage with the given stamps, it returns ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makePostage", "[", 
   RowBox[{"stampA_Integer", ",", "stampB_Integer", ",", "postage_Integer"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"a", "=", "0"}], ",", 
       RowBox[{"a", "\[LessEqual]", 
        RowBox[{"Floor", "[", 
         RowBox[{"postage", "/", "stampA"}], "]"}]}], ",", 
       RowBox[{"a", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"b", "=", "0"}], ",", 
         RowBox[{"b", "\[LessEqual]", 
          RowBox[{"Floor", "[", 
           RowBox[{"postage", "/", "stampB"}], "]"}]}], ",", 
         RowBox[{"b", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"stampA", "*", "a"}], "+", 
             RowBox[{"stampB", "*", "b"}]}], "\[Equal]", "postage"}], ",", 
           RowBox[{"Throw", "[", 
            RowBox[{"{", 
             RowBox[{"a", ",", "b"}], "}"}], "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "Applying ",
 StyleBox["makePostage", "Input"],
 " with stamps 4 and 5 and postage 13 tells us that it requires two 4-cent \
stamps and one 5 cent stamps to make 13 cents postage."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"makePostage", "[", 
  RowBox[{"4", ",", "5", ",", "13"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
On the other hand, it is not possible to make 11 cents postage with 4 and 5 \
cent stamps and so the function does not produce output.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"makePostage", "[", 
  RowBox[{"4", ",", "5", ",", "11"}], "]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Automating the Basis Step", "Subsubsection"],

Cell[TextData[{
 "The ",
 StyleBox["makePostage", "Input"],
 " function finds the number of stamps of each denomination needed to produce \
the desired postage. As such, it verifies individual basis step propositions. \
With a simple loop, we can verify all of the basis cases. Recall that the \
basis step was to verify that postage can be made for 12, 13, 14, and 15 \
cents."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"p", "=", "12"}], ",", 
   RowBox[{"p", "\[LessEqual]", "15"}], ",", 
   RowBox[{"p", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Print", "[", 
    RowBox[{"makePostage", "[", 
     RowBox[{"4", ",", "5", ",", "p"}], "]"}], "]"}]}], "\[IndentingNewLine]",
   "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "0"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "1"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "3"}], "}"}]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "We will make a function for this in a moment, but first observe that the \
number of propositions in the basis step is equal to the smaller denomination \
stamp. The reason for this is in the proof of the inductive step. You should \
review the second solution to Example 4 in the text. The key point is that to \
make postage of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "+", "1"}], TraditionalForm]]],
 " cents, the proof relies on the inductive assumption that you can make \
postage of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", "+", "1", "-", "4"}], "=", 
    RowBox[{"k", "-", "3"}]}], TraditionalForm]]],
 " cents. This requires that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", "-", "3"}], "\[GreaterEqual]", "12"}], TraditionalForm]]],
 ", or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[GreaterEqual]", "15"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Generically, if ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is the smaller of the stamps and ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is the minimum postage that we claim can be made (",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "12"}], TraditionalForm]]],
 " in the example), then the inductive step requires ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", "+", "1", "-", "a"}], "\[GreaterEqual]", "x"}], 
   TraditionalForm]]],
 ". Which is to say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[GreaterEqual]", 
    RowBox[{"x", "+", 
     RowBox[{"(", 
      RowBox[{"a", "-", "1"}], ")"}]}]}], TraditionalForm]]],
 ". Thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"P", "(", "x", ")"}], ",", 
    RowBox[{"P", "(", 
     RowBox[{"x", "+", "1"}], ")"}], ",", "\[Ellipsis]", ",", 
    RowBox[{"P", "(", 
     RowBox[{"x", "+", 
      RowBox[{"(", 
       RowBox[{"a", "-", "1"}], ")"}]}], ")"}]}], TraditionalForm]]],
 " must form the basis step. "
}], "Text"],

Cell["\<\
This is useful to us in the following way: given the values of the stamps and \
the minimum value of postage, we can automate the verification of the \
appropriate basis cases. \
\>", "Text"],

Cell[TextData[{
 "We first create the following message to be given in case one of the basis \
cases fails. Note the use of placeholders ",
 StyleBox["`1`", "Input"],
 ", ",
 StyleBox["`2`", "Input"],
 ", and ",
 StyleBox["`3`", "Input"],
 ". These serve as variables. When ",
 StyleBox[ButtonBox["Message",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Message"], "FunctionLink"],
 " is called, it will be given with three additional arguments, beyond the \
name of the message, and the values of these arguments will be substituted in \
for these variables."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"postageBasis", "::", "fail"}], "=", 
   "\"\<Cannot form postage of `3` using stamps of amounts `1` and \
`2`.\>\""}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Here is the ",
 StyleBox["postageBasis", "Input"],
 " function. Note that the expression ",
 StyleBox["postageList", "Input"],
 " terminates the first argument of ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 " is the second argument of ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 ". Consequently, if the main body of the function, contained in the first \
argument of ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 ", is successful, the result of the function will be the list of rules that \
specify the numbers of stamps needed for each amount of postage. If there is \
an amount of postage that is not possible, then the message will be generated \
and the result of the function will be ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"postageBasis", "[", 
   RowBox[{
   "stampA_Integer", ",", "stampB_Integer", ",", "minpostage_Integer"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"small", ",", 
      RowBox[{"postageList", "=", 
       RowBox[{"{", "}"}]}], ",", "postage", ",", "R"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"small", "=", 
      RowBox[{"Min", "[", 
       RowBox[{"stampA", ",", "stampB"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Check", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"postage", "=", "minpostage"}], ",", "\[IndentingNewLine]", 
          
          RowBox[{"postage", "\[LessEqual]", 
           RowBox[{"minpostage", "+", "small", "-", "1"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"postage", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"R", "=", 
            RowBox[{"makePostage", "[", 
             RowBox[{"stampA", ",", "stampB", ",", "postage"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"R", "=!=", "Null"}], ",", "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"postageList", ",", 
               RowBox[{"postage", "\[Rule]", "R"}]}], "]"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"Message", "[", 
              RowBox[{
               RowBox[{"postageBasis", "::", "fail"}], ",", "stampA", ",", 
               "stampB", ",", "postage"}], "]"}]}], "\[IndentingNewLine]", 
            "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", "postageList"}], ",", 
       RowBox[{"(*", 
        RowBox[{"end", " ", "of", " ", "Check", " ", "argument", " ", "1"}], 
        "*)"}], "\[IndentingNewLine]", "Null"}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We apply it to the example of using 4 and 5 cent stamps to make postage of at \
least 12 cents.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"postageBasis", "[", 
  RowBox[{"4", ",", "5", ",", "12"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"12", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"3", ",", "0"}], "}"}]}], ",", 
   RowBox[{"13", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"2", ",", "1"}], "}"}]}], ",", 
   RowBox[{"14", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}]}], ",", 
   RowBox[{"15", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "3"}], "}"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["postageBasis", "Input"],
 " function above accepts as input the denominations of the two stamps, and \
the minimum value such that all postage values equal to or greater than that \
minimum can be made. The function uses the ",
 StyleBox[ButtonBox["Min",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Min"], "FunctionLink"],
 " command to set ",
 StyleBox["small", "Input"],
 " equal to the lesser of the two denominations. The ",
 StyleBox["postageList", "Input"],
 " variable is used to store the information that shows how to make the \
various amounts of postage. This list will store rules of the form ",
 Cell[BoxData[
  RowBox[{
   StyleBox["postage", "CodeVar"], "\[Rule]", "{", 
   StyleBox["a", "CodeVar"], ",", 
   StyleBox["b", "CodeVar"], "}"}]], "myCode"],
 " which indicate that the specified amount of ",
 Cell[BoxData[
  FormBox[
   StyleBox["postage", "CodeVar"], TraditionalForm]]],
 " can be made with ",
 StyleBox["a", "CodeVar"],
 " stamps of value ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 StyleBox["b", "CodeVar"],
 " of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop considers each of the basis cases (using the observation above). For \
each amount of postage, we use ",
 StyleBox["makePostage", "Input"],
 " to determine if it is possible to form that postage from the stamp values. \
If so, ",
 StyleBox["makePostage", "Input"],
 " returns a pair indicating how the desired postage is made and this is \
added to the ",
 StyleBox["postageList", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 " evaluates its first argument and the output of the ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 " is the output of the first argument, unless a message had been raised, in \
which case the second argument is evaluated. If ",
 StyleBox["makePostage", "Input"],
 " ever returns ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", that indicates that the particular basis case cannot be verified. In this \
case, the message ",
 StyleBox["postageBasis::fail", "Input"],
 " is raised. If no messages are raised, that means that every amount of \
postage succeeded and ",
 StyleBox["postageList", "Input"],
 ", the last expression of the first argument to ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 " is output. "
}], "Text"],

Cell["\<\
The following indicates that it is false that every amount of postage of 12 \
cents or larger can be obtained using 4 and 6 cent stamps.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"postageBasis", "[", 
  RowBox[{"4", ",", "6", ",", "12"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"postageBasis", "::", "fail"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Cannot form postage of \[NoBreak]\\!\\(13\\)\[NoBreak] \
using stamps of amounts \[NoBreak]\\!\\(4\\)\[NoBreak] and \
\[NoBreak]\\!\\(6\\)\[NoBreak].\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"postageBasis", "::", "fail"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Cannot form postage of \[NoBreak]\\!\\(15\\)\[NoBreak] \
using stamps of amounts \[NoBreak]\\!\\(4\\)\[NoBreak] and \
\[NoBreak]\\!\\(6\\)\[NoBreak].\"\>"}]], "Message", "MSG"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["5.3 Recursive Definitions and Structural Induction", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 "In this section we will show how functions and sets can be defined \
recursively in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell["A Simple Recursive Function", "Subsection",
 CellTags->"Section3a"],

Cell[TextData[{
 "First we consider the recursively defined function from Example 1 in \
Section 5.3 of the text. This function is defined by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "0", ")"}], "=", "3"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"n", "+", "1"}], ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"f", "(", "n", ")"}]}], "+", "3"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "In order to represent this function in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we must transform the recursive part of the definition into an equation \
for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "n", ")"}], TraditionalForm]]],
 " instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", 
    RowBox[{"n", "+", "1"}], ")"}], TraditionalForm]]],
 ". We have to decrease all of the arguments in the recursive part of the \
definition by 1: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"f", "(", 
       RowBox[{"n", "-", "1"}], ")"}]}], "+", "3"}]}], TraditionalForm]]],
 ". The formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"n", "+", "1"}], ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"f", "(", "n", ")"}]}], "+", "3"}]}], TraditionalForm]]],
 " is perhaps more expressive in that the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " suggests that the definition is about the \[OpenCurlyDoubleQuote]next\
\[CloseCurlyDoubleQuote] value of the function. But ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " cannot interpret ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " as a parameter in a function definition, at least not in a natural way."
}], "Text"],

Cell[TextData[{
 "It is also important to point out that changing the recursive formula also \
changes the domain over which it is valid. The formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"n", "+", "1"}], ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"f", "(", "n", ")"}]}], "+", "3"}]}], TraditionalForm]]],
 " applies for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[GreaterEqual]", "0"}], TraditionalForm]]],
 ", while ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"f", "(", 
       RowBox[{"n", "-", "1"}], ")"}]}], "+", "3"}]}], TraditionalForm]]],
 " applies for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[GreaterEqual]", "1"}], TraditionalForm]]],
 ". This does not affect the value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "n", ")"}], TraditionalForm]]],
 " for any ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The most natural way to define a recursive function is to define the \
function just like any other. In this case, the function will refer to \
itself, but ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has no problem with that."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"2", "*", 
    RowBox[{"f", "[", 
     RowBox[{"n", "-", "1"}], "]"}]}], "+", "3"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We also must define the basis step. To declare ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "0", ")"}], "=", "3"}], TraditionalForm]]],
 ", you make the assignment as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "0", "]"}], "=", "3"}]], "Input",
 InitializationCell->True],

Cell[BoxData["3"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the recursive part of the definition must be given as a ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 "). The basis step can be defined with either ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ") or ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 ")."
}], "Text"],

Cell[TextData[{
 "Now that the recursive definition and the basis step have been assigned, \
you can use ",
 StyleBox["f", "Input"],
 " like any other function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", "10", "]"}]], "Input"],

Cell[BoxData["6141"], "Output"]
}, Open  ]],

Cell[TextData[{
 "And you can compute the values of ",
 StyleBox["f", "Input"],
 " from 1 to 9 using ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"f", "[", "n", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "9"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "9", ",", "21", ",", "45", ",", "93", ",", "189", ",", "381", ",", "765", 
   ",", "1533", ",", "3069"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Down Values", "Subsubsection"],

Cell[TextData[{
 "It is worth understanding a bit of what ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is doing when you define and evaluate a recursively defined function. Any \
time you evaluate a function, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " checks the function\[CloseCurlyQuote]s list of down values. We first \
discussed down values in Section 2.3. Briefly, the down values for a symbol \
are the expressions associated with the symbol when it is given arguments, as \
opposed to the symbol by itself.  Here are the down values for the function \
",
 StyleBox["f", "Input"],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "f", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f", "[", "0", "]"}], "]"}], "\[RuleDelayed]", "3"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"f", "[", "n_", "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"f", "[", 
       RowBox[{"n", "-", "1"}], "]"}]}], "+", "3"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This indicates that ",
 StyleBox["f", "Input"],
 " has two down values: one associated to the argument 0 and one to arguments \
matching the pattern ",
 StyleBox["n_", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Whenever you apply ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ") or ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 ") with the left hand side of the operator of the form ",
 StyleBox["f[\[Ellipsis]]", "Input"],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " understands this to be the creation or modification of one of ",
 StyleBox["f", "Input"],
 "\[CloseCurlyQuote]s down values. Note that such an assignment does not \
affect any down values other than the one, if any, whose left hand side is of \
exactly the same form. Consequently, when dealing with recursive functions, \
it is a good idea to apply ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " to a function name when redefining a function, as this will remove all the \
down values for the symbol."
}], "Text"],

Cell[TextData[{
 "When you evaluate ",
 StyleBox["f", "Input"],
 " at a particular value, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " checks the more specific down values first. That is, if you enter ",
 StyleBox["f[0]", "Input"],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will use the down value associated to ",
 StyleBox["f[0]", "Input"],
 " and output 3 rather than applying the formula ",
 StyleBox["2*f[0-1]+3", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "If you apply ",
 StyleBox["f", "Input"],
 " to 0, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " sees that 0 has a specific down value and returns the value. If you apply ",
 StyleBox["f", "Input"],
 " to a different value, say 2, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not find a specific down value for 2 and so it applies the formula. \
The formula says that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "2", ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"f", "(", "1", ")"}]}], "+", "3"}]}], TraditionalForm]]],
 ". When ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " tries to evaluate this, it recognizes that it needs to find ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "1", ")"}], TraditionalForm]]],
 ". Again, it checks the down values and, not finding a result for 1, it \
applies the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "1", ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"f", "(", "0", ")"}]}], "+", "3"}]}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "0", ")"}], TraditionalForm]]],
 " has a specific value, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " looks up ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "0", ")"}], TraditionalForm]]],
 ", which allows it to compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "1", ")"}], "=", "9"}], TraditionalForm]]],
 " and then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "2", ")"}], "=", "21"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "You may be wondering why the output above indicates that ",
 StyleBox["f", "Input"],
 "'s list of down values only has the value for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "0", ")"}], TraditionalForm]]],
 ", even though we've computed other values. The reason is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " doesn't automatically store down values. Most of the time, you don't \
execute functions on the same input multiple times, so it would be a waste of \
memory to store every value computed. You have to explicitly tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to store down values."
}], "Text"],

Cell[TextData[{
 "We have seen that you can store a specific value as a down value by making \
an assignment such as ",
 StyleBox["f[0]=3", "Input"],
 ". An assignment of this form can be given within the body of a function. \
Doing so causes the function to store values as it calculates them, as we\
\[CloseCurlyQuote]ll see below."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["A Second Recursive Function", "Subsection",
 CellTags->"Section3b"],

Cell[TextData[{
 "Now consider the function ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " defined by the basis values ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", "0", ")"}], "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", "1", ")"}], "=", "1"}], TraditionalForm]]],
 " and the recursive formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"F", "(", 
      RowBox[{"n", "-", "1"}], ")"}], "+", 
     RowBox[{"F", "(", 
      RowBox[{"n", "-", "2"}], ")"}]}]}], TraditionalForm]]],
 ". This is the function whose values are the Fibonacci numbers."
}], "Text"],

Cell["\<\
We will define this function as indicated above. First, we establish the \
basis values.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"F", "[", "0", "]"}], "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"F", "[", "1", "]"}], "=", "1"}], ";"}]}], "Input",
 InitializationCell->True],

Cell["Note that these have been stored as down values.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "F", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "0", "]"}], "]"}], "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "1", "]"}], "]"}], "\[RuleDelayed]", "1"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
For the recursive part of the definition, we enter the following.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"F", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"F", "[", "n", "]"}], "=", 
   RowBox[{
    RowBox[{"F", "[", 
     RowBox[{"n", "-", "1"}], "]"}], "+", 
    RowBox[{"F", "[", 
     RowBox[{"n", "-", "2"}], "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "This may look a bit strange, but it is actually fairly straightforward. The \
expression is an application of ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 "), which is an assignment to the symbol ",
 StyleBox["F", "Input"],
 " with argument matching the pattern ",
 StyleBox["n_", "Input"],
 ", i.e., with any argument. Note that this has created a single entry in the \
down values list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "F", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "0", "]"}], "]"}], "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "1", "]"}], "]"}], "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "n_", "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"F", "[", "n", "]"}], "=", 
      RowBox[{
       RowBox[{"F", "[", 
        RowBox[{"n", "-", "1"}], "]"}], "+", 
       RowBox[{"F", "[", 
        RowBox[{"n", "-", "2"}], "]"}]}]}], ")"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The \[OpenCurlyDoubleQuote]value\[CloseCurlyDoubleQuote] associated to ",
 StyleBox["F", "Input"],
 " with argument matching ",
 StyleBox["n_", "Input"],
 " is the expression ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"F", "[", "n", "]"}], "=", 
   RowBox[{"F", "[", 
    RowBox[{"n", "-", "1"}], "]"}], "+", 
   RowBox[{"F", "[", 
    RowBox[{"n", "-", "2"}], "]"}]}]], "myCode"],
 ". It can be useful to think of ",
 StyleBox["F", "Input"],
 " as representing both a function and a computer program. Assigning ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"F", "[", "0", "]"}], "=", "1"}]], "myCode"],
 " and ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"F", "[", "1", "]"}], "=", "1"}]], "myCode"],
 " establish the basis values for the function ",
 StyleBox["F", "Input"],
 ". Setting ",
 StyleBox["F[n_]", "Input"],
 " defines the action of the program ",
 StyleBox["F", "Input"],
 ". That program has a single line of code which is ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"F", "[", "n", "]"}], "=", 
   RowBox[{"F", "[", 
    RowBox[{"n", "-", "1"}], "]"}], "+", 
   RowBox[{"F", "[", 
    RowBox[{"n", "-", "2"}], "]"}]}]], "myCode"],
 ". That line of code in the program sets a particular value of the function ",
 StyleBox["F", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The function ",
 StyleBox["F", "Input"],
 " is now completely defined and produces correct output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"F", "[", "n", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "0", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "2", ",", "3", ",", "5", ",", "8", ",", "13", ",", "21",
    ",", "34", ",", "55", ",", "89"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also note that, unlike the function ",
 StyleBox["f", "Input"],
 " above, ",
 StyleBox["F", "Input"],
 " has added the results of its calculations to its down values."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "F", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "0", "]"}], "]"}], "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "1", "]"}], "]"}], "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "2", "]"}], "]"}], "\[RuleDelayed]", "2"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "3", "]"}], "]"}], "\[RuleDelayed]", "3"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "4", "]"}], "]"}], "\[RuleDelayed]", "5"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "5", "]"}], "]"}], "\[RuleDelayed]", "8"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "6", "]"}], "]"}], "\[RuleDelayed]", "13"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "7", "]"}], "]"}], "\[RuleDelayed]", "21"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "8", "]"}], "]"}], "\[RuleDelayed]", "34"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "9", "]"}], "]"}], "\[RuleDelayed]", "55"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "10", "]"}], "]"}], "\[RuleDelayed]", "89"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"F", "[", "n_", "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"F", "[", "n", "]"}], "=", 
      RowBox[{
       RowBox[{"F", "[", 
        RowBox[{"n", "-", "1"}], "]"}], "+", 
       RowBox[{"F", "[", 
        RowBox[{"n", "-", "2"}], "]"}]}]}], ")"}]}]}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Comparing Complexity", "Subsection",
 CellTags->"Section3c"],

Cell[TextData[{
 "The difference in time complexity between a recursive function that stores \
computed values and one that does not is very significant, perhaps much more \
than you might think. Let's create two new functions, both of which model the \
function ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", 
      RowBox[{"g", "(", 
       RowBox[{"n", "-", "1"}], ")"}]}], "+", 
     RowBox[{"3", "\[CenterDot]", 
      RowBox[{"g", "(", 
       RowBox[{"n", "-", "2"}], ")"}]}]}]}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "0", ")"}], "=", "5"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "1", ")"}], "=", "2"}], TraditionalForm]]],
 ". The function ",
 StyleBox["gR", "Input"],
 " will \[OpenCurlyDoubleQuote]remember\[CloseCurlyDoubleQuote] computed \
values while ",
 StyleBox["gF", "Input"],
 " will be \[OpenCurlyDoubleQuote]forgetful\[CloseCurlyDoubleQuote] and not \
store values."
}], "Text"],

Cell[TextData[{
 "We will track the recursive calls by adding a ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " statement every time the formula is used. Here are the two functions."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"gR", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Computing gR[\>\"", ",", "n", ",", "\"\<]\>\""}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"gR", "[", "n", "]"}], "=", 
      RowBox[{
       RowBox[{"2", "*", 
        RowBox[{"gR", "[", 
         RowBox[{"n", "-", "1"}], "]"}]}], "+", 
       RowBox[{"3", "*", 
        RowBox[{"gR", "[", 
         RowBox[{"n", "-", "2"}], "]"}]}]}]}]}]}], "\[IndentingNewLine]", 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"gR", "[", "0", "]"}], "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"gR", "[", "1", "]"}], "=", "2"}], ";"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[{
 RowBox[{
  RowBox[{"gF", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Computing gF[\>\"", ",", "n", ",", "\"\<]\>\""}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"2", "*", 
       RowBox[{"gF", "[", 
        RowBox[{"n", "-", "1"}], "]"}]}], "+", 
      RowBox[{"3", "*", 
       RowBox[{"gF", "[", 
        RowBox[{"n", "-", "2"}], "]"}]}]}]}]}], "\[IndentingNewLine]", 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"gF", "[", "0", "]"}], "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"gF", "[", "1", "]"}], "=", "2"}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Let\[CloseCurlyQuote]s see what happens when we attempt to compute ",
 StyleBox["gR[5]", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gR", "[", "5", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gR[\"\>", "\[InvisibleSpace]", "5", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gR[", 5, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gR[\"\>", "\[InvisibleSpace]", "4", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gR[", 4, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gR[\"\>", "\[InvisibleSpace]", "3", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gR[", 3, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gR[\"\>", "\[InvisibleSpace]", "2", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gR[", 2, "]"],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData["422"], "Output"]
}, Open  ]],

Cell[TextData[{
 "This appears fairly straightforward. We enter ",
 StyleBox["gR[5]", "Input"],
 ", and since this has not been stored as a down value, the formula is \
evaluated, causing the print statement to be executed. Then, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "5", ")"}], TraditionalForm]]],
 " depends on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "4", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "3", ")"}], TraditionalForm]]],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " needs to know, first, the value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "4", ")"}], TraditionalForm]]],
 ". Since this has not been previously computed, again the formula needs to \
be invoked causing ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to print \[OpenCurlyDoubleQuote]",
 StyleBox["Computing gR[4]", "Input"],
 "\[CloseCurlyDoubleQuote]. Computing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "4", ")"}], TraditionalForm]]],
 " requires the values for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "3", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "2", ")"}], TraditionalForm]]],
 ". Again, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "3", ")"}], TraditionalForm]]],
 " is needed first, so ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " executes ",
 StyleBox["gR[3]", "Input"],
 ". This requires ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "2", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "1", ")"}], TraditionalForm]]],
 ". To obtain the value for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "2", ")"}], TraditionalForm]]],
 ", the formula must be invoked once again. "
}], "Text"],

Cell[TextData[{
 "After printing \[OpenCurlyDoubleQuote]",
 StyleBox["Computing gR[2]", "Input"],
 "\[CloseCurlyDoubleQuote], ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " needs to evaluate the expression ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"gR", "[", "2", "]"}], "=", "2", "*", 
   RowBox[{"gR", "[", "1", "]"}], "+", "3", "*", 
   RowBox[{"gR", "[", "0", "]"}]}]], "myCode"],
 ". This time, it can look to the stored values of ",
 StyleBox["gR[1]", "Input"],
 " and ",
 StyleBox["gR[0]", "Input"],
 " to compute that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "2", ")"}], "=", "19"}], TraditionalForm]]],
 ". This value is stored as a down value of ",
 StyleBox["gR", "Input"],
 ", and we begin unwinding. ",
 StyleBox["gR[2]", "Input"],
 " was called when attempting to compute ",
 StyleBox["gR[3]", "Input"],
 ", which depends on values for 2 and 1. Since ",
 StyleBox["gR[2]", "Input"],
 " was just returned and ",
 StyleBox["gR[1]", "Input"],
 " is stored, ",
 StyleBox["gR[3]", "Input"],
 " can be computed and stored. The value of ",
 StyleBox["gR[3]", "Input"],
 " is sent back up to the computation of ",
 StyleBox["gR[4]", "Input"],
 ". ",
 StyleBox["gR[4]", "Input"],
 " depends on values of 3 and 2 and both of those values are now available, \
so ",
 StyleBox["gR[4]", "Input"],
 " can be computed. Likewise, ",
 StyleBox["gR[5]", "Input"],
 " can be determined using the values of ",
 StyleBox["gR[4]", "Input"],
 " and ",
 StyleBox["gR[3]", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Contrast this with what happens for ",
 StyleBox["gF[5]", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gF", "[", "5", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gF[\"\>", "\[InvisibleSpace]", "5", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gF[", 5, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gF[\"\>", "\[InvisibleSpace]", "4", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gF[", 4, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gF[\"\>", "\[InvisibleSpace]", "3", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gF[", 3, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gF[\"\>", "\[InvisibleSpace]", "2", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gF[", 2, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gF[\"\>", "\[InvisibleSpace]", "2", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gF[", 2, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gF[\"\>", "\[InvisibleSpace]", "3", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gF[", 3, "]"],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Computing gF[\"\>", "\[InvisibleSpace]", "2", 
   "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm["Computing gF[", 2, "]"],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData["422"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Things begin the same. We execute ",
 StyleBox["gF[5]", "Input"],
 " causing the print statement to report the fact that we are computing that \
value. To obtain ",
 StyleBox["gF[5]", "Input"],
 ", we need the values of ",
 StyleBox["gF[4]", "Input"],
 " and ",
 StyleBox["gF[3]", "Input"],
 ". ",
 StyleBox["gF[4]", "Input"],
 " is computed first, since it is leftmost in the formula and \
\[OpenCurlyDoubleQuote]",
 StyleBox["Computing gF[4]", "Input"],
 "\[CloseCurlyDoubleQuote] is displayed. To obtain the value for 4, we need \
the values for 3 and 2, and so ",
 StyleBox["gF[3]", "Input"],
 " is executed. This requires the values for ",
 StyleBox["gF[2]", "Input"],
 " and ",
 StyleBox["gF[1]", "Input"],
 ". Again, since it is leftmost, ",
 StyleBox["gF[2]", "Input"],
 " is tackled first, causing \[OpenCurlyDoubleQuote]Computing ",
 StyleBox["gF[2]", "Input"],
 "\[CloseCurlyDoubleQuote] to be displayed. "
}], "Text"],

Cell[TextData[{
 "This value can be computed from the stored values ",
 StyleBox["gF[1]", "Input"],
 " and ",
 StyleBox["gF[0]", "Input"],
 ". That means that ",
 StyleBox["gF[2]", "Input"],
 ", which was invoked in the attempt to compute ",
 StyleBox["gF[3]", "Input"],
 ", is complete and we move back up to the computation of ",
 StyleBox["gF[3]", "Input"],
 ". Since ",
 StyleBox["gF[3]", "Input"],
 " depends on the recently computed ",
 StyleBox["gF[2]", "Input"],
 " and on ",
 StyleBox["gF[1]", "Input"],
 ", which was stored as an initial condition, ",
 StyleBox["gF[3]", "Input"],
 " can be computed as well."
}], "Text"],

Cell[TextData[{
 "This means that ",
 StyleBox["gF[4]", "Input"],
 " now knows the value of ",
 StyleBox["gF[3]", "Input"],
 ". You can think about it as if the formula ",
 StyleBox["gF[4]", "Input"],
 " is working with is now ",
 Cell[BoxData[
  RowBox[{"2", "*", "44", "+", "3", "*", 
   RowBox[{"gF", "[", "2", "]"}]}]], "myCode"],
 ". ",
 StyleBox["gF[4]", "Input"],
 " now just needs the value of ",
 StyleBox["gF[2]", "Input"],
 ". ",
 StyleBox["gR", "Input"],
 " could just look that value up, since it had recorded the value ",
 StyleBox["gR[2]", "Input"],
 " when it was first computed. But ",
 StyleBox["gF", "Input"],
 " is not storing values that are computed, so it must apply the formula to \
compute ",
 StyleBox["gF[2]", "Input"],
 " again. Thus, \[OpenCurlyDoubleQuote]",
 StyleBox["Computing gF[2]", "Input"],
 "\[CloseCurlyDoubleQuote] is displayed a second time. Once the formula has \
been used to once again compute ",
 StyleBox["gF[2]", "Input"],
 ", then this value is available to ",
 StyleBox["gF[4]", "Input"],
 " to complete its computation."
}], "Text"],

Cell[TextData[{
 "Once ",
 StyleBox["gF[4]", "Input"],
 " has finished computing, that value is sent up. ",
 StyleBox["gF[5]", "Input"],
 ", which required ",
 StyleBox["gF[4]", "Input"],
 " and ",
 StyleBox["gF[3]", "Input"],
 ", now knows one of the two values it required. But again, ",
 StyleBox["gF[3]", "Input"],
 ", which was computed as part of ",
 StyleBox["gF[4]", "Input"],
 ", must be computed all over again. Consequently, you see the message \
\[OpenCurlyDoubleQuote]",
 StyleBox["Computing gF[3]", "Input"],
 "\[CloseCurlyDoubleQuote] again. And ",
 StyleBox["gF[3]", "Input"],
 " requires the value for 2, so ",
 StyleBox["gF[2]", "Input"],
 " must be executed a third time. "
}], "Text"],

Cell["\<\
As you can imagine, the difference between storing values and not is even \
more extreme for larger input values. When values are stored, once the \
recursion starts working its way back up the ladder, it remembers all the \
results from the lower values. When values are not stored, the chain of \
recursive calls has to keep recomputing the results from lower valued inputs.\
\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["A Recursive Function with Two Parameters", "Subsection",
 CellTags->"Section3d"],

Cell[TextData[{
 "In Example 13 of Section 5.3 of the text, the sequence ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"m", ",", "n"}]], TraditionalForm]]],
 " is defined. We will define a function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "(", 
    RowBox[{"m", ",", "n"}], ")"}], TraditionalForm]]],
 " that models the sequence ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"m", ",", "n"}]], TraditionalForm]]],
 ". The basis value is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", "(", 
     RowBox[{"0", ",", "0"}], ")"}], "=", "0"}], TraditionalForm]]],
 ", and the recursion formula is"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"A", "(", 
    RowBox[{"m", ",", "n"}], ")"}], "=", 
   TagBox[GridBox[{
      {"\[Piecewise]", GridBox[{
         {
          RowBox[{
           RowBox[{"A", "(", 
            RowBox[{
             RowBox[{"m", "-", "1"}], ",", "n"}], ")"}], "+", "1"}], 
          RowBox[{
           RowBox[{"if", " ", "n"}], "=", 
           RowBox[{
            RowBox[{"0", " ", "and", " ", "m"}], ">", "0"}]}]},
         {
          RowBox[{
           RowBox[{"A", "(", 
            RowBox[{"m", ",", 
             RowBox[{"n", "-", "1"}]}], ")"}], "+", "n"}], 
          RowBox[{
           RowBox[{"if", " ", "n"}], ">", "0"}]}
        },
        AllowedDimensions->{2, Automatic},
        Editable->True,
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxItemSize->{
         "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}},
           "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.84]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}},
        Selectable->True]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.35]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    "Piecewise",
    DeleteWithContents->True,
    Editable->False,
    SelectWithContents->True,
    Selectable->False]}], TraditionalForm]], "Equation"],

Cell["\<\
As with the previous example, we will define the function so as to store \
values. The initial value is as follows.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"A", "[", 
    RowBox[{"0", ",", "0"}], "]"}], "=", "0"}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We will define the recursive formula using a ",
 StyleBox[ButtonBox["Which",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Which"], "FunctionLink"],
 ". Recall that the ",
 StyleBox[ButtonBox["Which",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Which"], "FunctionLink"],
 " function accepts an even number of arguments, with the first in each pair \
a condition and the second in the pair evaluated if the condition holds."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A", "[", 
   RowBox[{"m_", ",", "n_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"A", "[", 
    RowBox[{"m", ",", "n"}], "]"}], "=", 
   RowBox[{"Which", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"n", "\[Equal]", "0"}], "&&", 
      RowBox[{"m", ">", "0"}]}], ",", 
     RowBox[{
      RowBox[{"A", "[", 
       RowBox[{
        RowBox[{"m", "-", "1"}], ",", "n"}], "]"}], "+", "1"}], ",", 
     RowBox[{"n", ">", "0"}], ",", 
     RowBox[{
      RowBox[{"A", "[", 
       RowBox[{"m", ",", 
        RowBox[{"n", "-", "1"}]}], "]"}], "+", "n"}]}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now we can compute some values of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"m", ",", "n"}]], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A", "[", 
  RowBox[{"3", ",", "2"}], "]"}]], "Input"],

Cell[BoxData["6"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A", "[", 
  RowBox[{"5", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["11"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To get a better idea of what the values of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"m", ",", "n"}]], TraditionalForm]]],
 " are, we can display a ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " of them using ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"A", "[", 
     RowBox[{"m", ",", "n"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"m", ",", "0", ",", "10"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "0", ",", "10"}], "}"}]}], "]"}], "//", 
  "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"0", "1", "3", "6", "10", "15", "21", "28", "36", "45", "55"},
    {"1", "2", "4", "7", "11", "16", "22", "29", "37", "46", "56"},
    {"2", "3", "5", "8", "12", "17", "23", "30", "38", "47", "57"},
    {"3", "4", "6", "9", "13", "18", "24", "31", "39", "48", "58"},
    {"4", "5", "7", "10", "14", "19", "25", "32", "40", "49", "59"},
    {"5", "6", "8", "11", "15", "20", "26", "33", "41", "50", "60"},
    {"6", "7", "9", "12", "16", "21", "27", "34", "42", "51", "61"},
    {"7", "8", "10", "13", "17", "22", "28", "35", "43", "52", "62"},
    {"8", "9", "11", "14", "18", "23", "29", "36", "44", "53", "63"},
    {"9", "10", "12", "15", "19", "24", "30", "37", "45", "54", "64"},
    {"10", "11", "13", "16", "20", "25", "31", "38", "46", "55", "65"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the first variable to be specified is shown in rows. That is, the \
first row corresponds to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", "0"}], TraditionalForm]]],
 ", the second row has ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", "1"}], TraditionalForm]]],
 ", etc.",
 Cell[BoxData[
  FormBox[Cell[""], TraditionalForm]]]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["A Recursively Defined Set", "Subsection",
 CellTags->"Section3e"],

Cell["\<\
In Example 5, the text describes how to recursively define a set. Here, we \
will consider a slightly more complicated example. \
\>", "Text"],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " be the subset of the integers defined by:"
}], "Text"],

Cell[TextData[{
 "Basis step: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "\[Element]", "S"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", "\[Element]", "S"}], TraditionalForm]]],
 "."
}], "Text",
 CellMargins->{{24, 10}, {3, 0}}],

Cell[TextData[{
 "Recursive step: if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[Element]", "S"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[Element]", "S"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "+", "y"}], "\[Element]", "S"}], TraditionalForm]]],
 "."
}], "Text",
 CellMargins->{{24, 10}, {3, 0}}],

Cell["\<\
(Note that this is the set of all postage that can be formed with 4 cent and \
7 cent stamps.)\
\>", "Text"],

Cell[TextData[{
 "To model ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we will define a list that includes the elements called for in the basis \
step. For the recursive step, we will define a function that applies the \
recursion to the set."
}], "Text"],

Cell[TextData[{
 "The basis step requires that 4 and 7 are members of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". So we define ",
 StyleBox["S", "Input"],
 " to be the list consisting of 4 and 7."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S", "=", 
  RowBox[{"{", 
   RowBox[{"4", ",", "7"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "7"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To implement the recursive step, we will create a function ",
 StyleBox["recurseS", "Input"],
 ". This function will accept as input the current ",
 StyleBox["S", "Input"],
 " and will return the list obtained after applying the recursive rule. For \
instance, in the first application of ",
 StyleBox["recurseS", "Input"],
 ", the procedure needs to add ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "+", "4"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "+", "7"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", "+", "4"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", "+", "7"}], TraditionalForm]]],
 ". (We will eliminate duplication by applying the ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " function.)"
}], "Text"],

Cell[TextData[{
 "We will use a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop. Recall that in a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop, the first argument is the expression that is evaluated. Subsequent \
arguments specify the loop variables. Also recall that a variable \
specification of the form ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["i", "CodeVar"], ",", 
   StyleBox["list", "CodeVar"], "}"}]], "myCode"],
 " causes ",
 StyleBox["i", "CodeVar"],
 " to take on each element of the ",
 StyleBox["list", "CodeVar"],
 "."
}], "Text"],

Cell["Here is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"recurseS", "[", "S_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", 
      RowBox[{"T", "=", "S"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"T", "=", 
        RowBox[{"Union", "[", 
         RowBox[{"T", ",", 
          RowBox[{"{", 
           RowBox[{"x", "+", "y"}], "}"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "S"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "S"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     "T"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now we apply this function to ",
 StyleBox["S", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S", "=", 
  RowBox[{"recurseS", "[", "S", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "7", ",", "8", ",", "11", ",", "14"}], "}"}]], "Output"]
}, Open  ]],

Cell["After a second iteration:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S", "=", 
  RowBox[{"recurseS", "[", "S", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "4", ",", "7", ",", "8", ",", "11", ",", "12", ",", "14", ",", "15", ",", 
   "16", ",", "18", ",", "19", ",", "21", ",", "22", ",", "25", ",", "28"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["A third:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S", "=", 
  RowBox[{"recurseS", "[", "S", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "4", ",", "7", ",", "8", ",", "11", ",", "12", ",", "14", ",", "15", ",", 
   "16", ",", "18", ",", "19", ",", "20", ",", "21", ",", "22", ",", "23", 
   ",", "24", ",", "25", ",", "26", ",", "27", ",", "28", ",", "29", ",", 
   "30", ",", "31", ",", "32", ",", "33", ",", "34", ",", "35", ",", "36", 
   ",", "37", ",", "38", ",", "39", ",", "40", ",", "41", ",", "42", ",", 
   "43", ",", "44", ",", "46", ",", "47", ",", "49", ",", "50", ",", "53", 
   ",", "56"}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["A Set of Strings", "Subsection",
 CellTags->"Section3f"],

Cell["\<\
As the final example in this section, we will look at how to generate sets of \
strings over a finite alphabet, as described in Definition 1 of Section 5.3. \
\
\>", "Text"],

Cell[TextData[{
 "The alphabet we will use is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], 
    "}"}], TraditionalForm]]],
 ". We begin by assigning this to a name."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alphabet", "=", 
  RowBox[{"{", 
   RowBox[{
   "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], 
   "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "According to Definition 1, the basis step is that our set of strings must \
contain the empty string. In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", the empty string is given by ",
 StyleBox["\"\"", "Input"],
 ". We will use the name ",
 StyleBox["S2", "Input"],
 " for our set of strings, since ",
 StyleBox["S", "Input"],
 " is used above."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S2", "=", 
  RowBox[{"{", "\"\<\>\"", "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", "\<\"\"\>", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that while the output makes this appear to be an empty list, applying ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " reveals that it does in fact contain the empty string."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S2", "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"List", "[", "\"\<\>\"", "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Like the previous example, we will create a function to build the set of \
strings. The recursive step in the definition tells us that we build the set \
by combining every element of ",
 StyleBox["S2", "Input"],
 " with every letter in the alphabet. Again we will use a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop with two loop variables. We will define the function to accept the \
current version of the set and the alphabet as arguments."
}], "Text"],

Cell[TextData[{
 "To combine two strings, we apply the ",
 StyleBox[ButtonBox["StringJoin",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringJoin"], "FunctionLink"],
 " (",
 StyleBox["<>", "Input"],
 ") operator. Here is the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"buildStrings", "[", 
   RowBox[{"S_List", ",", "A_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"T", "=", "S"}], ",", "w", ",", "x"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"T", "=", 
        RowBox[{"Union", "[", 
         RowBox[{"T", ",", 
          RowBox[{"{", 
           RowBox[{"w", "<>", "x"}], "}"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"w", ",", "S"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "A"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     "T"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
The first application of the recursion adds the alphabet to the set. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S2", "=", 
  RowBox[{"buildStrings", "[", 
   RowBox[{"S2", ",", "alphabet"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
   ",", "\<\"d\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Again, since ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not display the enclosing quotation marks of strings in its output, \
the initial comma in the output above is separating the empty string from \
\[OpenCurlyDoubleQuote]a\[CloseCurlyDoubleQuote]. The second application adds \
all the two-character strings."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S2", "=", 
  RowBox[{"buildStrings", "[", 
   RowBox[{"S2", ",", "alphabet"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"\"\>", ",", "\<\"a\"\>", ",", "\<\"aa\"\>", ",", "\<\"ab\"\>", 
   ",", "\<\"ac\"\>", ",", "\<\"ad\"\>", ",", "\<\"b\"\>", ",", "\<\"ba\"\>", 
   ",", "\<\"bb\"\>", ",", "\<\"bc\"\>", ",", "\<\"bd\"\>", ",", "\<\"c\"\>", 
   ",", "\<\"ca\"\>", ",", "\<\"cb\"\>", ",", "\<\"cc\"\>", ",", "\<\"cd\"\>",
    ",", "\<\"d\"\>", ",", "\<\"da\"\>", ",", "\<\"db\"\>", ",", "\<\"dc\"\>",
    ",", "\<\"dd\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell["The third application includes the three-character strings.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S2", "=", 
  RowBox[{"buildStrings", "[", 
   RowBox[{"S2", ",", "alphabet"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"\"\>", ",", "\<\"a\"\>", ",", "\<\"aa\"\>", ",", "\<\"aaa\"\>",
    ",", "\<\"aab\"\>", ",", "\<\"aac\"\>", ",", "\<\"aad\"\>", 
   ",", "\<\"ab\"\>", ",", "\<\"aba\"\>", ",", "\<\"abb\"\>", 
   ",", "\<\"abc\"\>", ",", "\<\"abd\"\>", ",", "\<\"ac\"\>", 
   ",", "\<\"aca\"\>", ",", "\<\"acb\"\>", ",", "\<\"acc\"\>", 
   ",", "\<\"acd\"\>", ",", "\<\"ad\"\>", ",", "\<\"ada\"\>", 
   ",", "\<\"adb\"\>", ",", "\<\"adc\"\>", ",", "\<\"add\"\>", 
   ",", "\<\"b\"\>", ",", "\<\"ba\"\>", ",", "\<\"baa\"\>", 
   ",", "\<\"bab\"\>", ",", "\<\"bac\"\>", ",", "\<\"bad\"\>", 
   ",", "\<\"bb\"\>", ",", "\<\"bba\"\>", ",", "\<\"bbb\"\>", 
   ",", "\<\"bbc\"\>", ",", "\<\"bbd\"\>", ",", "\<\"bc\"\>", 
   ",", "\<\"bca\"\>", ",", "\<\"bcb\"\>", ",", "\<\"bcc\"\>", 
   ",", "\<\"bcd\"\>", ",", "\<\"bd\"\>", ",", "\<\"bda\"\>", 
   ",", "\<\"bdb\"\>", ",", "\<\"bdc\"\>", ",", "\<\"bdd\"\>", 
   ",", "\<\"c\"\>", ",", "\<\"ca\"\>", ",", "\<\"caa\"\>", 
   ",", "\<\"cab\"\>", ",", "\<\"cac\"\>", ",", "\<\"cad\"\>", 
   ",", "\<\"cb\"\>", ",", "\<\"cba\"\>", ",", "\<\"cbb\"\>", 
   ",", "\<\"cbc\"\>", ",", "\<\"cbd\"\>", ",", "\<\"cc\"\>", 
   ",", "\<\"cca\"\>", ",", "\<\"ccb\"\>", ",", "\<\"ccc\"\>", 
   ",", "\<\"ccd\"\>", ",", "\<\"cd\"\>", ",", "\<\"cda\"\>", 
   ",", "\<\"cdb\"\>", ",", "\<\"cdc\"\>", ",", "\<\"cdd\"\>", 
   ",", "\<\"d\"\>", ",", "\<\"da\"\>", ",", "\<\"daa\"\>", 
   ",", "\<\"dab\"\>", ",", "\<\"dac\"\>", ",", "\<\"dad\"\>", 
   ",", "\<\"db\"\>", ",", "\<\"dba\"\>", ",", "\<\"dbb\"\>", 
   ",", "\<\"dbc\"\>", ",", "\<\"dbd\"\>", ",", "\<\"dc\"\>", 
   ",", "\<\"dca\"\>", ",", "\<\"dcb\"\>", ",", "\<\"dcc\"\>", 
   ",", "\<\"dcd\"\>", ",", "\<\"dd\"\>", ",", "\<\"dda\"\>", 
   ",", "\<\"ddb\"\>", ",", "\<\"ddc\"\>", ",", "\<\"ddd\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["A General Function", "Subsubsection"],

Cell["\<\
We can put this process all together in one function. Given a set of strings \
representing the alphabet and a positive integer indicating the number of \
iterations desired, the following function will return the set of strings \
obtained after the given number of iterations. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"allStrings", "[", 
   RowBox[{"A_List", ",", "n_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"S", "=", 
       RowBox[{"{", "\"\<\>\"", "}"}]}], ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "n"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"S", "=", 
        RowBox[{"buildStrings", "[", 
         RowBox[{"S", ",", "A"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "S"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Below, we apply this function to the alphabet consisting of the strings \
\[OpenCurlyDoubleQuote]ab\[CloseCurlyDoubleQuote] and \
\[OpenCurlyDoubleQuote]ba\[CloseCurlyDoubleQuote] (in discrete mathematics, \
an alphabet does not have to consist of individual letters).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"allStrings", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<ab\>\"", ",", "\"\<ba\>\""}], "}"}], ",", "4"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"\"\>", ",", "\<\"ab\"\>", ",", "\<\"abab\"\>", 
   ",", "\<\"ababab\"\>", ",", "\<\"abababab\"\>", ",", "\<\"abababba\"\>", 
   ",", "\<\"ababba\"\>", ",", "\<\"ababbaab\"\>", ",", "\<\"ababbaba\"\>", 
   ",", "\<\"abba\"\>", ",", "\<\"abbaab\"\>", ",", "\<\"abbaabab\"\>", 
   ",", "\<\"abbaabba\"\>", ",", "\<\"abbaba\"\>", ",", "\<\"abbabaab\"\>", 
   ",", "\<\"abbababa\"\>", ",", "\<\"ba\"\>", ",", "\<\"baab\"\>", 
   ",", "\<\"baabab\"\>", ",", "\<\"baababab\"\>", ",", "\<\"baababba\"\>", 
   ",", "\<\"baabba\"\>", ",", "\<\"baabbaab\"\>", ",", "\<\"baabbaba\"\>", 
   ",", "\<\"baba\"\>", ",", "\<\"babaab\"\>", ",", "\<\"babaabab\"\>", 
   ",", "\<\"babaabba\"\>", ",", "\<\"bababa\"\>", ",", "\<\"bababaab\"\>", 
   ",", "\<\"babababa\"\>"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["5.4 Recursive Algorithms", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "In this section we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to implement several different recursive algorithms. First, we will look \
at two different recursive implementations of modular exponentiation and \
compare their performance. Then we will contrast a recursive approach to \
computing factorial with an iterative approach. And finally, we will provide \
an implementation of merge sort."
}], "Text"],

Cell[CellGroupData[{

Cell["Modular Exponentiation", "Subsection",
 CellTags->"Section4a"],

Cell[TextData[{
 "Example 4 of Section 5.4 of the text describes two recursive approaches to \
computing ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", "n"], "mod", " ", "m"}], TraditionalForm]]],
 ". Both of these use the initial condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "0"], "mod", " ", "m"}], "=", "1"}], 
   TraditionalForm]]],
 ". The first approach is based on the fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "n"], "mod", " ", "m"}], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"b", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["b", 
          RowBox[{"n", "-", "1"}]], "mod", " ", "m"}], ")"}]}], ")"}], "mod", 
     " ", "m"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The second approach is based on the observation that for even exponents, we \
can compute via the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "n"], "mod", " ", "m"}], "=", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["b", 
         RowBox[{"n", "/", "2"}]], "mod", " ", "m"}], ")"}], "2"], " ", "mod",
      " ", "m"}]}], TraditionalForm]]],
 ". And if the exponent is odd, we can use the identity ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "n"], "mod", " ", "m"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["b", 
            RowBox[{"\[LeftFloor]", 
             RowBox[{"n", "/", "2"}], "\[RightFloor]"}]], "mod", " ", "m"}], 
          ")"}], "2"], "mod", " ", "m"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"b", " ", "mod", " ", "m"}], ")"}]}], " ", "mod", " ", "m"}]}],
    TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell["Approach 1", "Subsubsection"],

Cell[TextData[{
 "First we will implement exponentiation based on the initial condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "0"], "mod", " ", "m"}], "=", "1"}], 
   TraditionalForm]]],
 " and the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "n"], "mod", " ", "m"}], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"b", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["b", 
          RowBox[{"n", "-", "1"}]], "mod", " ", "m"}], ")"}]}], ")"}], "mod", 
     " ", "m"}]}], TraditionalForm]]],
 ". The function ",
 StyleBox["power1", "Input"],
 " will accept three arguments: the base ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", the exponent ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and the modulus ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
If the exponent is 0, then regardless of the base or the modulus, the \
function returns 1. We can specify that behavior as follows. Observe that it \
is not necessary to give names to the other two arguments, since their \
specific values are irrelevant.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"power1", "[", 
   RowBox[{"_Integer", ",", "0", ",", "_Integer"}], "]"}], ":=", 
  "1"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", you are allowed to provide multiple definitions for the same function, \
provided the arguments have different forms. If two definitions both match a \
particular set of arguments, then the definition that is issued first, and is \
thus first in the list of down values, is the one applied. However, it is \
generally a good idea to ensure that the two patterns do not overlap. \
Consequently, for the recursive part of ",
 StyleBox["power1", "Input"],
 ", we will explicitly insist that the exponent be positive."
}], "Text"],

Cell[TextData[{
 "If the exponent is greater than 0, then we compute the product of the base \
with the function applied to the same base and modulus but the power \
decreased by 1. To ensure that the exponent is in fact positive, we will \
impose a ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") by entering the operator ",
 StyleBox["/;", "Input"],
 " following by the proposition ",
 StyleBox["n>0", "Input"],
 " between the argument list and the ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 ") operator."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"power1", "[", 
    RowBox[{"b_Integer", ",", "n_Integer", ",", "m_Integer"}], "]"}], "/;", 
   RowBox[{"n", ">", "0"}]}], ":=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    RowBox[{"b", "*", 
     RowBox[{"power1", "[", 
      RowBox[{"b", ",", 
       RowBox[{"n", "-", "1"}], ",", "m"}], "]"}]}], ",", "m"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Notice that the down values of the symbol ",
 StyleBox["power1", "Input"],
 " now includes both definitions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "power1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"power1", "[", 
      RowBox[{"_Integer", ",", "0", ",", "_Integer"}], "]"}], "]"}], 
    "\[RuleDelayed]", "1"}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{
      RowBox[{"power1", "[", 
       RowBox[{"b_Integer", ",", "n_Integer", ",", "m_Integer"}], "]"}], "/;", 
      RowBox[{"n", ">", "0"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"Mod", "[", 
     RowBox[{
      RowBox[{"b", " ", 
       RowBox[{"power1", "[", 
        RowBox[{"b", ",", 
         RowBox[{"n", "-", "1"}], ",", "m"}], "]"}]}], ",", "m"}], "]"}]}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that we did not store computed values in this function. The reason for \
this is that each iteration of the function only depends on one other call to \
the function. This is in contrast with the ",
 StyleBox["gR", "Input"],
 " and ",
 StyleBox["gF", "Input"],
 " functions from the previous section. Those functions called themselves \
twice in each iteration. As a result, those functions made use of the same \
value multiple times. When deciding whether or not to store values, you must \
weigh its potential benefit for not unnecessarily repeating computations with \
the cost of storage requirements."
}], "Text"],

Cell[TextData[{
 "We can use ",
 StyleBox["power1", "Input"],
 " to compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "6"], "mod", " ", "7"}], TraditionalForm]]],
 " and compare the result to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s computation of the same expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"power1", "[", 
  RowBox[{"3", ",", "6", ",", "7"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{
   RowBox[{"3", "^", "6"}], ",", "7"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Approach 2", "Subsubsection"],

Cell[TextData[{
 "The second approach computes the power based on Algorithm 4 from Section \
5.4. For exponent 0, it returns 1, just as before. If the exponent is even, \
then the algorithm uses the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "n"], "mod", " ", "m"}], "=", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["b", 
         RowBox[{"n", "/", "2"}]], "mod", " ", "m"}], ")"}], "2"], " ", "mod",
      " ", "m"}]}], TraditionalForm]]],
 ", and for odd powers, it computes the power using the identity ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "n"], "mod", " ", "m"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["b", 
            RowBox[{"\[LeftFloor]", 
             RowBox[{"n", "/", "2"}], "\[RightFloor]"}]], "mod", " ", "m"}], 
          ")"}], "2"], "mod", " ", "m"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"b", " ", "mod", " ", "m"}], ")"}]}], " ", "mod", " ", "m"}]}],
    TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
Since there are three possibilities, 0, even, or odd, we give the definition \
in three parts. \
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"power2", "[", 
    RowBox[{"_Integer", ",", "0", ",", "_Integer"}], "]"}], ":=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"power2", "[", 
     RowBox[{"b_Integer", ",", "n_Integer", ",", "m_Integer"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"n", ">", "0"}], "&&", 
     RowBox[{"EvenQ", "[", "n", "]"}]}]}], ":=", 
   RowBox[{"Mod", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"power2", "[", 
       RowBox[{"b", ",", 
        RowBox[{"n", "/", "2"}], ",", "m"}], "]"}], "^", "2"}], ",", "m"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"power2", "[", 
     RowBox[{"b_Integer", ",", "n_Integer", ",", "m_Integer"}], "]"}], "/;", 
    RowBox[{
     RowBox[{"n", ">", "0"}], "&&", 
     RowBox[{"OddQ", "[", "n", "]"}]}]}], ":=", 
   RowBox[{"Mod", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Mod", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"power2", "[", 
          RowBox[{"b", ",", 
           RowBox[{"Floor", "[", 
            RowBox[{"n", "/", "2"}], "]"}], ",", "m"}], "]"}], "^", "2"}], 
        ",", "m"}], "]"}], "*", "b"}], ",", "m"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We apply ",
 StyleBox["power2", "Input"],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "6"], "mod", " ", "7"}], TraditionalForm]]],
 " as well."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"power2", "[", 
  RowBox[{"3", ",", "6", ",", "7"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparing Performance of the Functions", "Subsubsection"],

Cell["\<\
Now that we've implemented these two algorithms, let's compare their \
performance on a variety of input values. \
\>", "Text"],

Cell[TextData[{
 "We fix the base 3 and the modulus 7 and consider the exponents from 100 to \
1000. To compare the performance, we'll time the execution of each function \
on the exponents from 100 to 1000. We use ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " to build the two lists consisting of the exponent/time pairs. Recall that ",
 StyleBox[ButtonBox["Timing",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Timing"], "FunctionLink"],
 " returns a list consisting of the time taken and the result, so we apply \
the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator to extract the time."
}], "Text"],

Cell[TextData[{
 "In order to obtain results with exponents as large as 1000, we need to \
temporarily increase the variable ",
 StyleBox[ButtonBox["$RecursionLimit",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/$RecursionLimit"], "FunctionLink"],
 ". ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " normally restricts the depth to which a recursive algorithm can go. This \
is useful, as it helps prevent infinite loops and other potential \
computer-crashing events. In this case, we are certain that our function will \
not cause such problems and we need large exponents in order to have times \
large enough to be compared. We modify the recursion limit by setting the \
symbol ",
 StyleBox[ButtonBox["$RecursionLimit",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/$RecursionLimit"], "FunctionLink"],
 " to ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 ". Doing so within a ",
 StyleBox[ButtonBox["Block",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Block"], "FunctionLink"],
 " ensures that this is only a temporary modification."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"times1", "=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"$RecursionLimit", "=", "Infinity"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n", ",", 
         RowBox[{
          RowBox[{"Timing", "[", 
           RowBox[{"power1", "[", 
            RowBox[{"3", ",", "n", ",", "7"}], "]"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", "100", ",", "1000"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"times2", "=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"$RecursionLimit", "=", "Infinity"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n", ",", 
         RowBox[{
          RowBox[{"Timing", "[", 
           RowBox[{"power2", "[", 
            RowBox[{"3", ",", "n", ",", "7"}], "]"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", "100", ",", "1000"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[TextData[{
 "We have suppressed the output in the statements above, but now ",
 StyleBox["times1", "Input"],
 " and ",
 StyleBox["times2", "Input"],
 " contain the running times for ",
 StyleBox["power1", "Input"],
 " and ",
 StyleBox["power2", "Input"],
 ", respectively."
}], "Text"],

Cell[TextData[{
 "Now let\[CloseCurlyQuote]s graph the time results. The ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " function can be used to plot multiple sets of data by giving as its \
argument a list whose elements are the lists of data. Also, by placing the \
data lists inside of the ",
 StyleBox[ButtonBox["Tooltip",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tooltip"], "FunctionLink"],
 " function, you can have labels appear when you hover your mouse pointer \
over the graph."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"times1", ",", "times2"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<power1\>\"", ",", "\"\<power2\>\""}], "}"}]}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{}, {{
       RGBColor[0.24720000000000014`, 0.24, 0.6], 
       PointBox[CompressedData["
1:eJxdm3dck9cXxqNWizvOonXEOmpFLW7qfNy4caOAhiEgKoYddggJCQlEKlZQ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        "]]}, {
       RGBColor[0.6, 0.24, 0.4428931686004542], 
       PointBox[CompressedData["
1:eJxdm2+IbGUdxyft1qalY5EsYjGG1SRYU4ZMFPqAFismjpKw9//Ze+/uzt6/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        "]]}}, {}}, AspectRatio -> 
    NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> True, 
    AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, Method -> {}, 
    PlotRange -> {{0, 1000.}, {0, 0.002429}}, PlotRangeClipping -> True, 
    PlotRangePadding -> {{20., 20.}, {0.000048580000000000006`, 
     0.000048580000000000006`}}],
   TemplateBox[{"\"power1\"", "\"power2\""}, "PointLegend", 
    DisplayFunction -> (StyleBox[
      StyleBox[
       PaneBox[
        TagBox[
         GridBox[{{
            TagBox[
             GridBox[{{
                GraphicsBox[{{}, {
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.24720000000000014`, 0.24, 0.6]], {
                    Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.24720000000000014`, 0.24, 0.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                 ImageSize -> {10, 10}, PlotRangePadding -> None, 
                 ImagePadding -> 1, 
                 BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                GraphicsBox[{{}, {
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.6, 0.24, 0.4428931686004542]], {
                    Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.6, 0.24, 0.4428931686004542]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                 ImageSize -> {10, 10}, PlotRangePadding -> None, 
                 ImagePadding -> 1, 
                 BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
              GridBoxAlignment -> {
               "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
              AutoDelete -> False, 
              GridBoxDividers -> {
               "Columns" -> {{False}}, "Rows" -> {{False}}}, 
              GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}}, 
              GridBoxSpacings -> {"Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], 
             "Grid"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
          AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], "Grid"], 
        Alignment -> Left, AppearanceElements -> None, 
        ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> "ResizeToFit"], 
       LineIndent -> 0, StripOnInput -> False], {FontFamily -> "Times"}, 
      Background -> Automatic, StripOnInput -> False]& ), Editable -> True, 
    InterpretationFunction :> (RowBox[{"PointLegend", "[", 
       RowBox[{
         RowBox[{"{", 
           RowBox[{
             RowBox[{"Directive", "[", 
               RowBox[{"RGBColor", "[", 
                 RowBox[{"0.24720000000000014`", ",", "0.24`", ",", "0.6`"}], 
                 "]"}], "]"}], ",", 
             RowBox[{"Directive", "[", 
               RowBox[{"RGBColor", "[", 
                 RowBox[{"0.6`", ",", "0.24`", ",", "0.4428931686004542`"}], 
                 "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
           RowBox[{#, ",", #2}], "}"}], ",", 
         RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}], ",", 
         RowBox[{"LegendMarkers", "\[Rule]", "False"}]}], "]"}]& )]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This gives us a visual comparison of the time complexity of the two \
algorithms. You can see that the second approach significantly outperforms \
the first. It is left to the reader to compare the functions we created to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " function."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Recursion and Iteration", "Subsection",
 CellTags->"Section4b"],

Cell["\<\
In this subsection we compare recursive and iterative approaches for \
computing factorial. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Recursive Factorial", "Subsubsection"],

Cell[TextData[{
 "First we will implement Algorithm 1 from Section 5.4, a recursive algorithm \
for computing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "!"}], TraditionalForm]]],
 ". This function accepts a nonnegative integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as its input. If the input value is ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 ", the function returns ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 ". Otherwise, it multiplies ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " by the value of the function applied to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"factorialR", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"n", "\[Equal]", "0"}], ",", "1", ",", 
      RowBox[{"n", "*", 
       RowBox[{"factorialR", "[", 
        RowBox[{"n", "-", "1"}], "]"}]}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that, while we could have defined the function\[CloseCurlyQuote]s \
action on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[NotEqual]", "0"}], TraditionalForm]]],
 " without using the ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " structure, a ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " will be needed in the iterative version and in comparing performance of \
algorithms, it is important to keep as much of the implementations the same \
as possible."
}], "Text"],

Cell[TextData[{
 "We test this function on 10 and verify that it has the same result as ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s built-in operator."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"factorialR", "[", "10", "]"}]], "Input"],

Cell[BoxData["3628800"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"10", "!"}]], "Input"],

Cell[BoxData["3628800"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Iterative Factorial", "Subsubsection"],

Cell[TextData[{
 "We can implement factorial with an iterative algorithm as well. Our \
function will use a variable ",
 StyleBox["f", "Input"],
 ", initialized to 1, to store the value of the factorial. It will compute \
using a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop with a loop variable running from 1 to ",
 StyleBox["n", "Input"],
 ". Within the loop, ",
 StyleBox["f", "Input"],
 " will be multiplied by the current value of the loop variable."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"factorialI", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"f", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"f", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "n"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"f", "=", 
        RowBox[{"f", "*", "i"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "f"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "We again check to make sure the result is correct on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "10"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"factorialI", "[", "10", "]"}]], "Input"],

Cell[BoxData["3628800"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparing Recursion and Iteration", "Subsubsection"],

Cell["\<\
Note that these two algorithms require exactly the same number of \
multiplications. From this point of view, their complexity is the same. \
\>", "Text"],

Cell[TextData[{
 "However, let's look at their performance. We consider values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " from 1 to 1500. We use the same approach as we did for ",
 StyleBox["power1", "Input"],
 " and ",
 StyleBox["power2", "Input"],
 " above to record and plot the time performance. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"timesR", "=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"$RecursionLimit", "=", "Infinity"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n", ",", 
         RowBox[{
          RowBox[{"Timing", "[", 
           RowBox[{"factorialR", "[", "n", "]"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", "1500"}], "}"}]}], "]"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"timesI", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"n", ",", 
       RowBox[{
        RowBox[{"Timing", "[", 
         RowBox[{"factorialI", "[", "n", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "1500"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"timesR", ",", "timesI"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<recursive\>\"", ",", "\"\<iterative\>\""}], "}"}]}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{}, {{
       RGBColor[0.24720000000000014`, 0.24, 0.6], 
       PointBox[CompressedData["
1:eJxdnHlczdn/xy9iQsgeY7n2LWQP4YUQQghlvUKS0K6Uuu23/e411mvPYGRs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        "]]}, {
       RGBColor[0.6, 0.24, 0.4428931686004542], 
       PointBox[CompressedData["
1:eJxdnHlczNsbxwch5ArxC2Hs2bNnfxBCGBQhTElCZdpXNe1T0zJ72cfFVVfI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        "]]}}, {}}, AspectRatio -> 
    NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> True, 
    AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, Method -> {}, 
    PlotRange -> {{0, 1500.}, {0, 0.004308}}, PlotRangeClipping -> True, 
    PlotRangePadding -> {{30., 30.}, {0.00008616, 0.00008616}}],
   TemplateBox[{"\"recursive\"", "\"iterative\""}, "PointLegend", 
    DisplayFunction -> (StyleBox[
      StyleBox[
       PaneBox[
        TagBox[
         GridBox[{{
            TagBox[
             GridBox[{{
                GraphicsBox[{{}, {
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.24720000000000014`, 0.24, 0.6]], {
                    Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.24720000000000014`, 0.24, 0.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                 ImageSize -> {10, 10}, PlotRangePadding -> None, 
                 ImagePadding -> 1, 
                 BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                GraphicsBox[{{}, {
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.6, 0.24, 0.4428931686004542]], {
                    Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0.6, 0.24, 0.4428931686004542]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                 ImageSize -> {10, 10}, PlotRangePadding -> None, 
                 ImagePadding -> 1, 
                 BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
              GridBoxAlignment -> {
               "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
              AutoDelete -> False, 
              GridBoxDividers -> {
               "Columns" -> {{False}}, "Rows" -> {{False}}}, 
              GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}}, 
              GridBoxSpacings -> {"Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], 
             "Grid"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
          AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], "Grid"], 
        Alignment -> Left, AppearanceElements -> None, 
        ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> "ResizeToFit"], 
       LineIndent -> 0, StripOnInput -> False], {FontFamily -> "Times"}, 
      Background -> Automatic, StripOnInput -> False]& ), Editable -> True, 
    InterpretationFunction :> (RowBox[{"PointLegend", "[", 
       RowBox[{
         RowBox[{"{", 
           RowBox[{
             RowBox[{"Directive", "[", 
               RowBox[{"RGBColor", "[", 
                 RowBox[{"0.24720000000000014`", ",", "0.24`", ",", "0.6`"}], 
                 "]"}], "]"}], ",", 
             RowBox[{"Directive", "[", 
               RowBox[{"RGBColor", "[", 
                 RowBox[{"0.6`", ",", "0.24`", ",", "0.4428931686004542`"}], 
                 "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
           RowBox[{#, ",", #2}], "}"}], ",", 
         RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}], ",", 
         RowBox[{"LegendMarkers", "\[Rule]", "False"}]}], "]"}]& )]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output"]
}, Open  ]],

Cell[TextData[{
 "First, you might wonder about the outliers which appear to be values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " for which one function or the other performs particularly poorly. These \
are essentially noise resulting from other processes on the computer. They \
are also inconsistent; running the commands again will produce slightly \
different results. "
}], "Text"],

Cell[TextData[{
 "However, it is clear that, despite the occasional peculiar value, the \
iterative function outperforms the recursive one for large values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". This is in spite of the fact that the two algorithms involve the same \
number of multiplications. Let's consider other sources of potential \
differences in complexity."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop in the iterative function includes a comparison (",
 StyleBox["i", "Input"],
 " must be tested against ",
 StyleBox["n", "Input"],
 " to determine if the loop continues) and the recursive function makes the \
comparison in the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement. So the two functions are effectively equal in terms of the \
number of comparisons used."
}], "Text"],

Cell[TextData[{
 "The iterative function involves two assignments (the assignment of ",
 StyleBox["f", "Input"],
 " and the implicit assignment of ",
 StyleBox["i", "Input"],
 " when it is incremented) absent from the recursive version. But two \
assignments are much less costly than a function call. Recursive function \
calls require ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to perform several operations in the background, both in order to execute \
the recursive call and to keep track of where ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is in the chain of recursive calls. All of which takes time and memory \
that the iterative approach does not require."
}], "Text"],

Cell["\<\
It is important to keep in mind the cost of recursion. When an iterative \
algorithm is available, it may be more efficient. On the other hand, \
recursive algorithms are often more natural to use and can better reveal the \
mathematical concepts. \
\>", "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Merge Sort", "Subsection",
 CellTags->"Section4c"],

Cell["We conclude this section by implementing merge sort.", "Text"],

Cell[TextData[{
 "The merge sort algorithm is described in Algorithm 9 of the text. The ",
 StyleBox["mergeSort", "Input"],
 " function will accept as its input a list of integers. Its main function is \
to split the single list it receives into two halves, unless the list \
contains only one element. The output of ",
 StyleBox["mergeSort", "Input"],
 " is the result of applying ",
 StyleBox["mergeSort", "Input"],
 " to both halves of the list and then recombining them with ",
 StyleBox["merge", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The following implements Algorithm 9. Note that the ",
 StyleBox["merge", "Input"],
 " function will be written next. For now, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " simply accepts ",
 StyleBox["merge", "Input"],
 " as a symbol."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mergeSort", "[", 
   RowBox[{"L", ":", 
    RowBox[{"{", "__Integer", "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"m", ",", "L1", ",", "L2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Length", "[", "L", "]"}], ">", "1"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"m", "=", 
        RowBox[{"Floor", "[", 
         RowBox[{
          RowBox[{"Length", "[", "L", "]"}], "/", "2"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"L1", "=", 
        RowBox[{"L", "[", 
         RowBox[{"[", 
          RowBox[{";;", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"L2", "=", 
        RowBox[{"L", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"m", "+", "1"}], ";;"}], "]"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"merge", "[", 
        RowBox[{
         RowBox[{"mergeSort", "[", "L1", "]"}], ",", 
         RowBox[{"mergeSort", "[", "L2", "]"}]}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", "L"}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "Note the use of the ",
 StyleBox[ButtonBox["Span",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Span"], "FunctionLink"],
 " (",
 StyleBox[";;", "Input"],
 ") operator to determine the two sublists. The syntax ",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "CodeVar"], ";;", 
   StyleBox["b", "CodeVar"]}]], "myCode"],
 ", within the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator, is used to obtain the sublist of elements from position ",
 StyleBox["a", "CodeVar"],
 " to ",
 StyleBox["b", "CodeVar"],
 ". Omitting ",
 StyleBox["a", "CodeVar"],
 ", as in ",
 StyleBox[";;5", "Input"],
 ", obtains the elements from the beginning to position 5. And omitting ",
 StyleBox["b", "CodeVar"],
 ", as in ",
 StyleBox["3;;", "Input"],
 ", obtains the elements from position 3 to the end of the list."
}], "Text"],

Cell[CellGroupData[{

Cell["Implementing Merge", "Subsubsection"],

Cell[TextData[{
 "To complete the function, we need to define ",
 StyleBox["merge", "Input"],
 ". The ",
 StyleBox["merge", "Input"],
 " function accepts two lists, which it assumes are sorted, and returns a \
single list that contains all the elements of both of the inputs and is \
sorted. The procedure is described in Algorithm 10 of the text."
}], "Text"],

Cell[TextData[{
 "In implementing ",
 StyleBox["merge", "Input"],
 ", the first step is to duplicate the input lists. This is because the lists \
are emptied of their elements as the ",
 StyleBox["merge", "Input"],
 " proceeds, and arguments to a function cannot be modified. We also \
initialize the list that will be the result to the empty list."
}], "Text"],

Cell[TextData[{
 "The main work of ",
 StyleBox["merge", "Input"],
 " is contained in a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop conditioned on both lists being nonempty. Within this ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop there are an ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement and a ",
 StyleBox[ButtonBox["Which",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Which"], "FunctionLink"],
 ". The ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement will implement the instruction to \[OpenCurlyDoubleQuote]remove \
smaller of first elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " from its list; put it at the right end of ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 "\[CloseCurlyDoubleQuote] from Algorithm 10. This ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement will test whether the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is smaller than the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 ". If so, then the first element is removed from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and added to ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ". If not, in the else clause, the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is moved to ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We use ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 " to add elements to the end of the list ",
 StyleBox["L", "Input"],
 ". Note that ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 ", when applied to the name of a list and an element, automatically replaces \
the list stored in the symbol. We use ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 " to remove the first element from the list. ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 " is applied to a list and a position and returns the list with the element \
in that position removed. Unlike ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 " does not automatically update a symbol representing a list, so we must \
reassign the name of the list to the result of the ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Which",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Which"], "FunctionLink"],
 " statement will implement the if statement found in Algorithm 10. The first \
condition will be that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is empty, and if so the remainder of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " will be added to ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " will be emptied. In a second condition, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " will be tested. Note that the ",
 StyleBox[ButtonBox["Which",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Which"], "FunctionLink"],
 " is ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s answer to the else-if construction. ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " is used to combine two lists."
}], "Text"],

Cell[TextData[{
 "Here is the implementation of ",
 StyleBox["merge", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"merge", "[", 
   RowBox[{
    RowBox[{"l1", ":", 
     RowBox[{"{", "__Integer", "}"}]}], ",", 
    RowBox[{"l2", ":", 
     RowBox[{"{", "__Integer", "}"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"L", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"L1", "=", "l1"}], ",", 
      RowBox[{"L2", "=", "l2"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"L1", "=!=", 
          RowBox[{"{", "}"}]}], ")"}], "&&", 
        RowBox[{"(", 
         RowBox[{"L2", "=!=", 
          RowBox[{"{", "}"}]}], ")"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"L1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "<", 
           RowBox[{"L2", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"L", ",", 
             RowBox[{"L1", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"L1", "=", 
            RowBox[{"Delete", "[", 
             RowBox[{"L1", ",", "1"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
          
          RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"L", ",", 
             RowBox[{"L2", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"L2", "=", 
            RowBox[{"Delete", "[", 
             RowBox[{"L2", ",", "1"}], "]"}]}]}]}], "\[IndentingNewLine]", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Which", "[", 
         RowBox[{
          RowBox[{"L1", "===", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"L", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"L", ",", "L2"}], "]"}]}], ";", 
           RowBox[{"L2", "=", 
            RowBox[{"{", "}"}]}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"L2", "===", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"L", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"L", ",", "L1"}], "]"}]}], ";", 
           RowBox[{"L1", "=", 
            RowBox[{"{", "}"}]}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "L"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We apply ",
 StyleBox["mergeSort", "Input"],
 " to a list as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mergeSort", "[", 
  RowBox[{"{", 
   RowBox[{"7", ",", "4", ",", "1", ",", "5", ",", "2", ",", "3", ",", "6"}], 
   "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7"}], 
  "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tracing Merge Sort", "Subsubsection"],

Cell[TextData[{
 "To better understand how functions work, it is sometimes useful to create \
\[OpenCurlyDoubleQuote]verbose\[CloseCurlyDoubleQuote] versions of them. That \
is, versions of the functions that use ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " or other means to report on what it is that they are doing. We provide a \
verbose version of ",
 StyleBox["mergeSort", "Input"],
 " and apply it to the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "2"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mergeSortVerbose", "[", 
   RowBox[{"L", ":", 
    RowBox[{"{", "__Integer", "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"m", ",", "L1", ",", "L2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", 
      RowBox[{"\"\<mergeSort called on \>\"", ",", "L"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "L", "]"}], ">", "1"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"m", "=", 
         RowBox[{"Floor", "[", 
          RowBox[{
           RowBox[{"Length", "[", "L", "]"}], "/", "2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<m=\>\"", ",", "m"}], "]"}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"L1", "=", 
         RowBox[{"L", "[", 
          RowBox[{"[", 
           RowBox[{";;", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<L1=\>\"", ",", "L1"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"L2", "=", 
         RowBox[{"L", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"m", "+", "1"}], ";;"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<L2=\>\"", ",", "L2"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"merge", "[", 
         RowBox[{
          RowBox[{"mergeSortVerbose", "[", "L1", "]"}], ",", 
          RowBox[{"mergeSortVerbose", "[", "L2", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Print", "[", "\"\<length=1\>\"", "]"}], ";", 
        "\[IndentingNewLine]", "L"}]}], "\[IndentingNewLine]", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mergeSortVerbose", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "1", ",", "2"}], "}"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"mergeSort called on \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "2"}], "}"}]}],
  SequenceForm["mergeSort called on ", {3, 1, 2}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"m=\"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["m=", 1],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"L1=\"\>", "\[InvisibleSpace]", 
   RowBox[{"{", "3", "}"}]}],
  SequenceForm["L1=", {3}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"L2=\"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}]}],
  SequenceForm["L2=", {1, 2}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"mergeSort called on \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", "3", "}"}]}],
  SequenceForm["mergeSort called on ", {3}],
  Editable->False]], "Print"],

Cell[BoxData["\<\"length=1\"\>"], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"mergeSort called on \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}]}],
  SequenceForm["mergeSort called on ", {1, 2}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"m=\"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["m=", 1],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"L1=\"\>", "\[InvisibleSpace]", 
   RowBox[{"{", "1", "}"}]}],
  SequenceForm["L1=", {1}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"L2=\"\>", "\[InvisibleSpace]", 
   RowBox[{"{", "2", "}"}]}],
  SequenceForm["L2=", {2}],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"mergeSort called on \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", "1", "}"}]}],
  SequenceForm["mergeSort called on ", {1}],
  Editable->False]], "Print"],

Cell[BoxData["\<\"length=1\"\>"], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"mergeSort called on \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", "2", "}"}]}],
  SequenceForm["mergeSort called on ", {2}],
  Editable->False]], "Print"],

Cell[BoxData["\<\"length=1\"\>"], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We recommend reading through the output. Then try it with a larger example, \
say with 7 elements, to make sure that you understand how merge sort works. \
You can also create a verbose version of ",
 StyleBox["merge", "Input"],
 " to see the details of how that function works."
}], "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["5.5 Program Correctness", "Section",
 CellTags->"Section5"],

Cell[TextData[{
 "In this section we will prove the correctness of the merge sort program \
that we implemented in the last section. This will require that we prove the \
correctness of ",
 StyleBox["merge", "Input"],
 " as well as ",
 StyleBox["mergeSort", "Input"],
 ". We begin with ",
 StyleBox["merge", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["merge", "Input"]], "Subsection",
 CellTags->"Section5a"],

Cell[TextData[{
 "For convenience, we will repeat the definition of ",
 StyleBox["merge", "Input"],
 ". Also, we've added comments to indicate where we've broken the procedure \
into three segments: ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"merge", "[", 
   RowBox[{
    RowBox[{"l1", ":", 
     RowBox[{"{", "__Integer", "}"}]}], ",", 
    RowBox[{"l2", ":", 
     RowBox[{"{", "__Integer", "}"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"L", ",", "L1", ",", "L2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", 
     RowBox[{"begin", " ", "S1"}], "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"L", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"L1", "=", "l1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"L2", "=", "l2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"end", " ", "S1"}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"L1", "=!=", 
          RowBox[{"{", "}"}]}], ")"}], "&&", 
        RowBox[{"(", 
         RowBox[{"L2", "=!=", 
          RowBox[{"{", "}"}]}], ")"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{"begin", " ", "S2"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"L1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "<", 
           RowBox[{"L2", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"L", ",", 
             RowBox[{"L1", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"L1", "=", 
            RowBox[{"Delete", "[", 
             RowBox[{"L1", ",", "1"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
          
          RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"L", ",", 
             RowBox[{"L2", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"L2", "=", 
            RowBox[{"Delete", "[", 
             RowBox[{"L2", ",", "1"}], "]"}]}]}]}], "\[IndentingNewLine]", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"end", " ", "S2"}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"begin", " ", "S3"}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"Which", "[", 
         RowBox[{
          RowBox[{"L1", "===", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"L", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"L", ",", "L2"}], "]"}]}], ";", 
           RowBox[{"L2", "=", 
            RowBox[{"{", "}"}]}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"L2", "===", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"L", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"L", ",", "L1"}], "]"}]}], ";", 
           RowBox[{"L1", "=", 
            RowBox[{"{", "}"}]}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{"end", " ", "S3"}], "*)"}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "L"}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " be the assertion that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 " (the inputs to the function) are ordered, nonempty, and disjoint lists of \
integers. Let ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " be the proposition that ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " (the output) is an ordered list and that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"L", "=", 
    RowBox[{
     SubscriptBox["l", "1"], "\[Union]", 
     SubscriptBox["l", "2"]}]}], TraditionalForm]]],
 " as sets (that is, the set of integers appearing in ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is equal to the union of the set of integers appearing in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 " and the set of integers in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 "). We claim that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     StyleBox["merge", "Input"], "}"}], "q"}], TraditionalForm]]],
 ", i.e., that ",
 StyleBox["merge", "Input"],
 " is partially correct with respect to the initial condition ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and the final assertion ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "1"], TraditionalForm]]],
 " be the proposition that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "1"], "=", 
    SubscriptBox["l", "1"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "2"], "=", 
    SubscriptBox["l", "2"]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is the empty list. It is clear that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", 
     RowBox[{"{", 
      SubscriptBox["S", "1"], "}"}], "p"}], "\[And]", 
    SubscriptBox["q", "1"]}], TraditionalForm]]],
 "."
}], "Text"],

Cell["Define the following propositional variables:", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 " is the proposition that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 "; that is, the set of elements appearing in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is a subset of the set of elements appearing in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 ", and the order of the elements in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is the same as their order in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 ". Note that it immediately follows from ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 " that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is ordered."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]]],
 " is the proposition that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 " (and thus ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is ordered)."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "3"], TraditionalForm]]],
 " is the assertion that ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is ordered."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "4"], TraditionalForm]]],
 " is the assertion that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"L", "\[Union]", 
     SubscriptBox["L", "1"], "\[Union]", 
     SubscriptBox["L", "2"]}], "=", 
    RowBox[{
     SubscriptBox["l", "1"], "\[Union]", 
     SubscriptBox["l", "2"]}]}], TraditionalForm]]],
 " as sets."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "5"], TraditionalForm]]],
 " is the proposition that all members of ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " are smaller than all members of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 ". That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[ForAll]", 
      RowBox[{"x", "\[Element]", "L"}]}], ")"}], 
    RowBox[{"(", 
     RowBox[{"\[ForAll]", 
      RowBox[{"y", "\[Element]", 
       RowBox[{
        SubscriptBox["L", "1"], "\[Union]", 
        SubscriptBox["L", "2"]}]}]}], ")"}], 
    RowBox[{"(", 
     RowBox[{"x", "<", "y"}], ")"}]}], TraditionalForm]]],
 "."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["r", "1"], "\[And]", 
    SubscriptBox["r", "2"], "\[And]", 
    SubscriptBox["r", "3"], "\[And]", 
    SubscriptBox["r", "4"], "\[And]", 
    SubscriptBox["r", "5"]}], TraditionalForm]]],
 "."
}], "Item1"]
}, Open  ]],

Cell[TextData[{
 "We claim that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "\[And]", 
     SubscriptBox["q", "1"]}], "\[Rule]", "r"}], TraditionalForm]]],
 ". Assume ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[And]", 
    SubscriptBox["q", "1"]}], TraditionalForm]]],
 ". That is, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 " are ordered, nonempty, and disjoint lists of integers. Also, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "1"], "=", 
    SubscriptBox["l", "1"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "2"], "=", 
    SubscriptBox["l", "2"]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is empty. Then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]]],
 " hold since a list is a sublist of itself. Proposition ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "3"], TraditionalForm]]],
 " holds since ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is empty and thus is ordered vacuously. That ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "4"], TraditionalForm]]],
 " is true follows from substituting ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["\[EmptySet]", TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ". And ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "5"], TraditionalForm]]],
 " is vacuous since ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is empty. From ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", 
     RowBox[{"{", 
      SubscriptBox["S", "1"], "}"}], "p"}], "\[And]", 
    SubscriptBox["q", "1"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "\[And]", 
      SubscriptBox["q", "1"]}], ")"}], "\[Rule]", "r"}], TraditionalForm]]],
 ", we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     SubscriptBox["S", "1"], "}"}], "r"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Next, we will show that ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is a loop invariant for the loop ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"while", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["L", "1"], "\[NotEqual]", 
       RowBox[{
        RowBox[{"{", "}"}], " ", "and", " ", 
        SubscriptBox["L", "2"]}], "\[NotEqual]", 
       RowBox[{"{", "}"}]}], ")"}], 
     SubscriptBox["S", "2"]}], ";", 
    SubscriptBox["S", "3"]}], TraditionalForm]]],
 ". Denote by ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " the condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["L", "1"], "\[NotEqual]", 
     RowBox[{"{", "}"}]}], "\[And]", 
    RowBox[{
     SubscriptBox["L", "2"], "\[NotEqual]", 
     RowBox[{"{", "}"}]}]}], TraditionalForm]]],
 ". We must show that if ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " hold, then ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is true after ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "2"], ";", 
    SubscriptBox["S", "3"]}], TraditionalForm]]],
 " is executed. First we will show ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[And]", 
    RowBox[{"c", 
     RowBox[{"{", 
      SubscriptBox["S", "2"], "}"}], "r"}]}], TraditionalForm]]],
 " and then that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", 
    RowBox[{"{", 
     SubscriptBox["S", "3"], "}"}], "r"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "To show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[And]", 
    RowBox[{"c", 
     RowBox[{"{", 
      SubscriptBox["S", "2"], "}"}], "r"}]}], TraditionalForm]]],
 ", assume ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[And]", "c"}], TraditionalForm]]],
 ". That is, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is ordered, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"L", "\[Union]", 
     SubscriptBox["L", "1"], "\[Union]", 
     SubscriptBox["L", "2"]}], "=", 
    RowBox[{
     SubscriptBox["l", "1"], "\[Union]", 
     SubscriptBox["l", "2"]}]}], TraditionalForm]]],
 ", and all members of ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " are smaller than every member of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 ". Also, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " are nonempty. Then both ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " have first elements. Assume that the if condition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 " holds. That is, the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is smaller than the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 ". Then the two commands in the then clause of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 " are executed: the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is added to the end of ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " has its first element removed."
}], "Text"],

Cell[TextData[{
 "We need to show that ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " holds following the execution of the then clause of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 ": the new ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is a sublist of the old ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " since an element was removed meaning that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"1", "new"}]], "\[Subset]", 
    SubscriptBox["L", 
     RowBox[{"1", "old"}]]}], TraditionalForm]]],
 " as sets and the order of the remaining elements was not modified. Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "new"}]], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "old"}]], TraditionalForm]]],
 " which was a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 ", the new ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 "."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]]],
 ": the new ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is identical is the old ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " and thus remains a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 "."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "3"], TraditionalForm]]],
 ": the old ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " was ordered, and, since we assume that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "5"], TraditionalForm]]],
 " held before execution of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 ", every element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "old"], TraditionalForm]]],
 " was smaller than every element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"1", "old"}]], "\[Union]", 
    SubscriptBox["L", 
     RowBox[{"2", "old"}]]}], TraditionalForm]]],
 ". In particular, the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "old"}]], TraditionalForm]]],
 " was larger than all elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "old"], TraditionalForm]]],
 ". And so ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "new"], TraditionalForm]]],
 " is ordered."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "4"], TraditionalForm]]],
 ": The smallest element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " was removed from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and added to ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ". Thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["L", "new"], "\[Union]", 
     SubscriptBox["L", 
      RowBox[{"1", "new"}]]}], "=", 
    RowBox[{
     SubscriptBox["L", "old"], "\[Union]", 
     SubscriptBox["L", 
      RowBox[{"1", "old"}]]}]}], TraditionalForm]]],
 " and hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"L", "\[Union]", 
     SubscriptBox["L", "1"], "\[Union]", 
     SubscriptBox["L", "2"]}], "=", 
    RowBox[{
     SubscriptBox["l", "1"], "\[Union]", 
     SubscriptBox["l", "2"]}]}], TraditionalForm]]],
 "."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "5"], TraditionalForm]]],
 ": We must show that all members of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "new"], TraditionalForm]]],
 " are smaller than all members of both ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "new"}]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"2", "new"}]], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be an arbitrary element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "new"], TraditionalForm]]],
 ". Either ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " was a member of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "old"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " was the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "old"}]], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " was a member of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "old"], TraditionalForm]]],
 " then the assumption that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "5"], TraditionalForm]]],
 " held before execution of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 " guarantees that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is smaller than all elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"1", "new"}]], "\[Union]", 
    SubscriptBox["L", 
     RowBox[{"2", "new"}]]}], TraditionalForm]]],
 ". On the other hand, assume ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " was the first element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "old"}]], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "old"}]], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 ", it is ordered and thus ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " was also the smallest element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "old"}]], TraditionalForm]]],
 " and hence is less than all elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"1", "new"}]], TraditionalForm]]],
 ". Also, by the assumption that the if condition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 " evaluated true, ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is smaller than the first (and smallest) element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"2", "old"}]], "=", 
    SubscriptBox["L", 
     RowBox[{"2", "new"}]]}], TraditionalForm]]],
 " as well. So ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is smaller than all members of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", 
     RowBox[{"1", "new"}]], "\[Union]", 
    SubscriptBox["L", 
     RowBox[{"2", "new"}]]}], TraditionalForm]]],
 "."
}], "Item1"]
}, Open  ]],

Cell[TextData[{
 "The above shows that if the if condition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "2"], TraditionalForm]]],
 " holds, then ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " holds after executing the then clause. In case the condition fails and the \
else clause executes, the proof is similar. We conclude that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[And]", 
    RowBox[{"c", 
     RowBox[{"{", 
      SubscriptBox["S", "2"], "}"}], "r"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Next we will show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", 
    RowBox[{"{", 
     SubscriptBox["S", "3"], "}"}], "r"}], TraditionalForm]]],
 ". Assume ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " holds. Consider the case that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is empty. Then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is appended onto the end of ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is set to the empty list."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 ": ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " is, since we assume the if condition, empty and thus a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "1"], TraditionalForm]]],
 "."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]]],
 ": the second statement in the then clause sets ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " equal to the empty list, which is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 "."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "3"], TraditionalForm]]],
 ": we assume that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "old"], TraditionalForm]]],
 " is ordered, that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"2", "old"}]], TraditionalForm]]],
 " is a sublist of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["l", "2"], TraditionalForm]]],
 " and thus is ordered, and that all members of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "old"], TraditionalForm]]],
 " are smaller than all members of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"2", "old"}]], TraditionalForm]]],
 ". Thus, adding ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", 
    RowBox[{"2", "old"}]], TraditionalForm]]],
 " to the end of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "old"], TraditionalForm]]],
 " to produce ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "new"], TraditionalForm]]],
 " results in an ordered list."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "4"], TraditionalForm]]],
 ": as sets, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["L", "new"], "=", 
    RowBox[{
     SubscriptBox["L", "old"], "\[Union]", 
     SubscriptBox["L", 
      RowBox[{"2", "old"}]]}]}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["L", "new"], "\[Union]", 
     SubscriptBox["L", 
      RowBox[{"1", "new"}]], "\[Union]", 
     SubscriptBox["L", 
      RowBox[{"2", "new"}]]}], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["L", "old"], "\[Union]", 
       SubscriptBox["L", 
        RowBox[{"2", "old"}]]}], ")"}], "\[Union]", 
     SubscriptBox["L", 
      RowBox[{"1", "old"}]], "\[Union]", "\[EmptySet]"}]}], TraditionalForm]]],
 ". This is equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["l", "1"], "\[Union]", 
    SubscriptBox["l", "2"]}], TraditionalForm]]],
 " by the assumption that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "4"], TraditionalForm]]],
 " held before execution."
}], "Item1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "5"], TraditionalForm]]],
 ": after execution of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ", both ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " are empty and assertion ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "5"], TraditionalForm]]],
 " is true vacuously."
}], "Item1"]
}, Open  ]],

Cell[TextData[{
 "The case that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " is empty is similar. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", 
    RowBox[{"{", 
     SubscriptBox["S", "3"], "}"}], "r"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We have shown ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[And]", 
    RowBox[{"c", 
     RowBox[{"{", 
      SubscriptBox["S", "2"], "}"}], "r"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", 
    RowBox[{"{", 
     SubscriptBox["S", "3"], "}"}], "r"}], TraditionalForm]]],
 ", and thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[And]", 
    RowBox[{"c", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["S", "2"], ";", 
       SubscriptBox["S", "3"]}], "}"}], "r"}]}], TraditionalForm]]],
 ". Hence ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is a loop invariant for the while loop. By the inference rule for while \
loops, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"while", " ", "c", " ", 
       SubscriptBox["S", "2"]}], ";", 
      SubscriptBox["S", "3"]}], "}"}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Not]", "c"}], "\[And]", "r"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Combining this result with the conclusion that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     SubscriptBox["S", "1"], "}"}], "r"}], TraditionalForm]]],
 " from the paragraph immediately following the definition of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     StyleBox["merge", "Input"], "}"}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Not]", "c"}], "\[And]", "r"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We conclude by claiming ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Not]", "c"}], "\[And]", "r"}], "\[Rule]", "q"}], 
   TraditionalForm]]],
 ". Recall that ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " was the final assertion that ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is ordered and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"L", "=", 
    RowBox[{
     SubscriptBox["l", "1"], "\[Union]", 
     SubscriptBox["l", "2"]}]}], TraditionalForm]]],
 " as sets. Assume ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Not]", "c"}], "\[And]", "r"}], TraditionalForm]]],
 ". That ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is ordered is the claim of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "3"], TraditionalForm]]],
 ". By ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "4"], TraditionalForm]]],
 ", we have that, as sets, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"L", "\[Union]", 
     SubscriptBox["L", "1"], "\[Union]", 
     SubscriptBox["L", "2"]}], "=", 
    RowBox[{
     SubscriptBox["l", "1"], "\[Union]", 
     SubscriptBox["l", "2"]}]}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Not]", "c"}], TraditionalForm]]],
 " implies that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " are empty. Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"L", "=", 
    RowBox[{
     SubscriptBox["l", "1"], "\[Union]", 
     SubscriptBox["l", "2"]}]}], TraditionalForm]]],
 ". Thus ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " holds and we have completed the proof that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     StyleBox["merge", "Input"], "}"}], "q"}], TraditionalForm]]],
 " and hence ",
 StyleBox["merge", "Input"],
 " is partially correct."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["mergeSort", "Input"]], "Subsection",
 CellTags->"Section5b"],

Cell[TextData[{
 "Now we turn to the ",
 StyleBox["mergeSort", "Input"],
 " function. We repeat its definition below."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mergeSort", "[", 
   RowBox[{"L", ":", 
    RowBox[{"{", "__Integer", "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"m", ",", "L1", ",", "L2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Length", "[", "L", "]"}], ">", "1"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"m", "=", 
        RowBox[{"Floor", "[", 
         RowBox[{
          RowBox[{"Length", "[", "L", "]"}], "/", "2"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"L1", "=", 
        RowBox[{"L", "[", 
         RowBox[{"[", 
          RowBox[{";;", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"L2", "=", 
        RowBox[{"L", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"m", "+", "1"}], ";;"}], "]"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"merge", "[", 
        RowBox[{
         RowBox[{"mergeSort", "[", "L1", "]"}], ",", 
         RowBox[{"mergeSort", "[", "L2", "]"}]}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", "L"}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " be the assertion that ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is a nonempty list of distinct integers, and let ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " be the assertion that the function returns a list which has the same \
elements as ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " and is ordered. Our claim is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     StyleBox["mergeSort", "Input"], "}"}], "q"}], TraditionalForm]]],
 ". Since ",
 StyleBox["mergeSort", "Input"],
 " is recursive, our proof will be by strong induction on the length of the \
list ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "For the basis case, assume that ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " has only one element. Also assume ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Then the if condition of ",
 StyleBox["mergeSort", "Input"],
 " fails and the program terminates by returning ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " unmodified. But since ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " has only one element, it is trivially ordered. Thus, under the basis \
assumption that ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " has only one element, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     StyleBox["mergeSort", "Input"], "}"}], "q"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "For the inductive case, we make the inductive assumption that for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[LessEqual]", "n"}], TraditionalForm]]],
 ", if a list has length ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " then ",
 StyleBox["mergeSort", "Input"],
 " returns the list sorted. Assume ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " has ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " elements. Also assume ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Under these assumptions, the if condition is satisfied."
}], "Text"],

Cell[TextData[{
 "The first command in the then clause assigns ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{"\[LeftFloor]", 
     FractionBox[
      RowBox[{"n", "+", "1"}], "2"], "\[RightFloor]"}]}], TraditionalForm]]],
 ". Note that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "<", 
    RowBox[{"n", "+", "1"}]}], TraditionalForm]]],
 " and, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", ">", "1"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", ">", "0"}], TraditionalForm]]],
 ". All of the inequalities are strict."
}], "Text"],

Cell[TextData[{
 "The next two commands assign ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " to the list consisting of the first ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " elements in ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " to the remainder. Note that since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "<", "m", "<", 
    RowBox[{"n", "+", "1"}]}], TraditionalForm]]],
 ", both of these lists are nonempty with at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements."
}], "Text"],

Cell[TextData[{
 "In the final statement of the if clause, ",
 StyleBox["mergeSort", "Input"],
 " is applied to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 ". Since these two lists both have length at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", the inductive assumption implies that the results of ",
 StyleBox["mergeSort", "Input"],
 " on ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "2"], TraditionalForm]]],
 " are lists with the same elements and ordered. Since ",
 StyleBox["merge", "Input"],
 " is partially correct, as shown in the previous subsection, the result of \
",
 StyleBox["merge", "Input"],
 " is an ordered list consisting of the elements of its input lists. Hence, \
the result of ",
 StyleBox["mergeSort", "Input"],
 " is an ordered list consisting of the same elements as ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ". That is, ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " holds."
}], "Text"],

Cell[TextData[{
 "This concludes the inductive step and we conclude ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", 
    RowBox[{"{", 
     StyleBox["mergeSort", "Input"], "}"}], "q"}], TraditionalForm]]],
 " for all lengths of ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ". Hence, ",
 StyleBox["mergeSort", "Input"],
 " is partially correct."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 2", "Subsection",
 CellTags->"SolutionsCP2"],

Cell[TextData[{
 "Generate all well-formed formulae for expressions involving the variables \
",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " and the operators \:fffc ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"+", 
     RowBox[{",", 
      RowBox[{"\[CenterDot]", 
       RowBox[{",", 
        RowBox[{"/", 
         RowBox[{",", "-"}]}]}]}]}]}], "}"}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " or fewer symbols."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " This problem asks us to not only generate well-formed formulae, but to \
generate ",
 StyleBox["all",
  FontSlant->"Italic"],
 " such formulae subject to a limitation on the number of symbols."
}], "Text"],

Cell["\<\
To begin, we present a recursive definition of the set of well-formed \
formulae on the symbols.\
\>", "Text"],

Cell[TextData[{
 "Basis step: ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " are well-formed formulae."
}], "Text",
 CellMargins->{{24, 10}, {3, 0}}],

Cell[TextData[{
 "Recursive step: If ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are well-formed formulae, then so are: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"-", "F"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"F", "+", "G"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"F", "-", "G"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"F", "\[CenterDot]", "G"}], ")"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"F", "/", "G"}], ")"}], TraditionalForm]]],
 "."
}], "Text",
 CellMargins->{{24, 10}, {3, 0}}],

Cell["\<\
Note that we will fully parenthesize the well-formed formulae so as to avoid \
ambiguity, but parentheses will not be considered symbols for the purpose of \
counting symbols in the formula.\
\>", "Text"],

Cell[TextData[{
 "Also note that we will implement the well-formed formulae as strings, not \
as algebraic expressions. The reason for this is that if we build algebraic \
expressions, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will perform unwanted simplification. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{"-", "x"}], ")"}]}], TraditionalForm]]],
 " is a well-formed formula distinct from ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", but if we enter ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{"-", "x"}], ")"}]}], TraditionalForm]]],
 " as a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expression, it will be simplified to ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We will approach this problem in two steps. First, we generate well-formed \
formulae using a function with sufficiently many applications of the \
recursive step to guarantee that every well-formed formula of length ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " or less is produced. Second, we will prune the well-formed formulae with \
greater than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " symbols. This will leave us with all well-formed formulae involving at \
most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " symbols."
}], "Text"],

Cell[CellGroupData[{

Cell["Generating Formulae", "Subsubsection"],

Cell[TextData[{
 "The first step is to generate well-formed formulae. For this, we will \
create a pair of functions, similar to ",
 StyleBox["allStrings", "Input"],
 " and ",
 StyleBox["buildStrings", "Input"],
 " from Section 5.3 of this manual."
}], "Text"],

Cell[TextData[{
 "The function ",
 StyleBox["buildWFFs", "Input"],
 " will accept a single argument, a list of well-formed formulae. It will \
apply the recursive step to the existing set. The function first makes a copy \
of the input set, since arguments cannot be modified. Second, using a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop over the input set, it applies unary negation. Then, with a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop over the input set and over the binary operations, the function adds \
the rest of the well-formed formulae."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"buildWFFs", "[", "S_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"T", "=", "S"}], ",", "f", ",", "g", ",", "o"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"T", "=", 
        RowBox[{"Union", "[", 
         RowBox[{"T", ",", 
          RowBox[{"{", 
           RowBox[{"\"\<(-\>\"", "<>", "f", "<>", "\"\<)\>\""}], "}"}]}], 
         "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"f", ",", "S"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"T", "=", 
        RowBox[{"Union", "[", 
         RowBox[{"T", ",", 
          RowBox[{"{", 
           RowBox[{
           "\"\<(\>\"", "<>", "f", "<>", "o", "<>", "g", "<>", "\"\<)\>\""}], 
           "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"f", ",", "S"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"g", ",", "S"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"o", ",", 
         RowBox[{"{", 
          RowBox[{
          "\"\<+\>\"", ",", "\"\<-\>\"", ",", "\"\<*\>\"", ",", "\"\</\>\""}],
           "}"}]}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", "T"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Let's confirm that this works as expected by applying it to the basis set \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\""}], "}"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"buildWFFs", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\""}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"(-x)\"\>", ",", "\<\"x\"\>", ",", "\<\"(x*x)\"\>", 
   ",", "\<\"(x-x)\"\>", ",", "\<\"(x+x)\"\>", ",", "\<\"(x/x)\"\>", 
   ",", "\<\"(x*y)\"\>", ",", "\<\"(x-y)\"\>", ",", "\<\"(x+y)\"\>", 
   ",", "\<\"(x/y)\"\>", ",", "\<\"(x*z)\"\>", ",", "\<\"(x-z)\"\>", 
   ",", "\<\"(x+z)\"\>", ",", "\<\"(x/z)\"\>", ",", "\<\"(-y)\"\>", 
   ",", "\<\"y\"\>", ",", "\<\"(y*x)\"\>", ",", "\<\"(y-x)\"\>", 
   ",", "\<\"(y+x)\"\>", ",", "\<\"(y/x)\"\>", ",", "\<\"(y*y)\"\>", 
   ",", "\<\"(y-y)\"\>", ",", "\<\"(y+y)\"\>", ",", "\<\"(y/y)\"\>", 
   ",", "\<\"(y*z)\"\>", ",", "\<\"(y-z)\"\>", ",", "\<\"(y+z)\"\>", 
   ",", "\<\"(y/z)\"\>", ",", "\<\"(-z)\"\>", ",", "\<\"z\"\>", 
   ",", "\<\"(z*x)\"\>", ",", "\<\"(z-x)\"\>", ",", "\<\"(z+x)\"\>", 
   ",", "\<\"(z/x)\"\>", ",", "\<\"(z*y)\"\>", ",", "\<\"(z-y)\"\>", 
   ",", "\<\"(z+y)\"\>", ",", "\<\"(z/y)\"\>", ",", "\<\"(z*z)\"\>", 
   ",", "\<\"(z-z)\"\>", ",", "\<\"(z+z)\"\>", ",", "\<\"(z/z)\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the order is not the order in which the well-formed formulae are \
added, it is the order that ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " imposes on the set."
}], "Text"],

Cell[TextData[{
 "The other component is the function that calls ",
 StyleBox["buildWFFs", "Input"],
 ". This is nearly identical to ",
 StyleBox["allStrings", "Input"],
 ". ",
 StyleBox["allWFFs", "Input"],
 " accepts a positive integer ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " representing the number of applications of the recursive step that are to \
be performed. It initializes the set of formulae to the basis set and applies \
",
 StyleBox["buildWFFs", "Input"],
 " as many times as is called for."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"allWFFs", "[", "m_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"S", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"S", "=", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\""}], "}"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "m"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"S", "=", 
        RowBox[{"buildWFFs", "[", "S", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", "S"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now the question is: how many applications of the recursive step are needed \
to be sure that the result contains all well-formed formulae of length at \
most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "? Clearly, 0 applications of ",
 StyleBox["buildWFFs", "Input"],
 " are needed to obtain all formulae consisting of 1 symbol, as this is the \
basis step. Also, the formulae produced by an application of ",
 StyleBox["buildWFFs", "Input"],
 " contain at least one symbol more than was present in the previous step \
(from the unary negation). So after ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " applications of ",
 StyleBox["buildWFFs", "Input"],
 ", we are guaranteed to have all well-formed formulae with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " symbols or fewer."
}], "Text"],

Cell[TextData[{
 "To illustrate, we will find all well-formed formulae of length at most 3. \
Apply ",
 StyleBox["AllWFFs", "Input"],
 " to 2."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"allWFF3", "=", 
   RowBox[{"allWFFs", "[", "2", "]"}]}], ";"}]], "Input"],

Cell["We suppressed the output since the output would be lengthy.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "allWFF3", "]"}]], "Input"],

Cell[BoxData["7101"], "Output"]
}, Open  ]],

Cell["Here is the list of every 300th formula. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"allWFF3", "[", 
    RowBox[{"[", "i", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "7101", ",", "300"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"(-(-x))\"\>", ",", "\<\"((x-x)-(-y))\"\>", 
   ",", "\<\"((x*x)*(z-x))\"\>", ",", "\<\"((x-y)+(x-x))\"\>", 
   ",", "\<\"((x*y)-(y-y))\"\>", ",", "\<\"((x/y)*(z-y))\"\>", 
   ",", "\<\"((x*z)+(x-z))\"\>", ",", "\<\"((x/z)-(y-z))\"\>", 
   ",", "\<\"((-y)+(x+x))\"\>", ",", "\<\"(y-(x+y))\"\>", 
   ",", "\<\"((y+x)*(z+x))\"\>", ",", "\<\"((y/y)+(x+x))\"\>", 
   ",", "\<\"((y+y)-(y+y))\"\>", ",", "\<\"((y-y)*(z+z))\"\>", 
   ",", "\<\"((y+z)+(x+z))\"\>", ",", "\<\"((y-z)+(-z))\"\>", 
   ",", "\<\"((z+x)/x)\"\>", ",", "\<\"((z-x)-(y/x))\"\>", 
   ",", "\<\"((z*x)*(z/y))\"\>", ",", "\<\"((z-y)+(x/y))\"\>", 
   ",", "\<\"((z*y)-(y/z))\"\>", ",", "\<\"((z/y)*(z/z))\"\>", 
   ",", "\<\"((-z)+(z/y))\"\>", ",", "\<\"(z-(z/z))\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that these involve up to seven symbols. Since we want the formulae with \
at most three symbols, we must remove from this set all the formulae with \
more than three. For this, we will need a function that calculates the number \
of symbols in a formula. \
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pruning the Set", "Subsubsection"],

Cell[TextData[{
 "To count the number of characters in a string, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the ",
 StyleBox[ButtonBox["StringLength",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringLength"], "FunctionLink"],
 " function. If you apply ",
 StyleBox[ButtonBox["StringLength",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringLength"], "FunctionLink"],
 " to a string, the command returns the number of characters in the string."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringLength", "[", "\"\<abcde\>\"", "]"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[TextData[{
 "However, the number of symbols in a well-formed formula is not equal to its \
length, since parentheses are not considered symbols. Fortunately, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides the function ",
 StyleBox[ButtonBox["StringCount",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringCount"], "FunctionLink"],
 ". This function takes two arguments. The first is the string. The second is \
a substring or a list of substrings that you wish to count within the first. \
For example, to determine the number of a\[CloseCurlyQuote]s and b\
\[CloseCurlyQuote]s within the string \[OpenCurlyDoubleQuote]abcccbabbcab\
\[CloseCurlyDoubleQuote], you enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringCount", "[", 
  RowBox[{"\"\<abcccbabbcab\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\""}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["8"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To count the number of symbols in a well-formed formula, we apply ",
 StyleBox[ButtonBox["StringCount",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringCount"], "FunctionLink"],
 " with second argument the list of all symbols: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\"", ",", "\"\<+\>\"", ",", 
     "\"\<-\>\"", ",", "\"\<*\>\"", ",", "\"\</\>\""}], "}"}], 
   TraditionalForm]]],
 ". For example, the number of symbols in ",
 Cell[BoxData[
  FormBox["\"\<((z-x)-(z*x))\>\"", TraditionalForm]]],
 " is:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringCount", "[", 
  RowBox[{"\"\<((z-x)-(z*x))\>\"", ",", 
   RowBox[{"{", 
    RowBox[{
    "\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\"", ",", "\"\<+\>\"", ",", 
     "\"\<-\>\"", ",", "\"\<*\>\"", ",", "\"\</\>\""}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData["7"], "Output"]
}, Open  ]],

Cell[TextData[{
 "In order to prune the set ",
 StyleBox["allWFF3", "Input"],
 " so that it contains only the formulae with 3 or fewer symbols, we\
\[CloseCurlyQuote]ll use the ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " function can be used to find the subset of a given set consisting of those \
elements satisfying a given condition. ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " requires two arguments. The first is the original list. The second is a \
boolean-valued function of one argument that can be applied to the elements \
of the list and returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " for those elements that should be selected. The result is the list of \
elements of the original list for which the function returned true. "
}], "Text"],

Cell[TextData[{
 "In our case, the Boolean-valued function should return true if the \
well-formed formula has three or fewer symbols. We will create a function to \
test the result of ",
 StyleBox[ButtonBox["StringCount",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringCount"], "FunctionLink"],
 " against 3."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"testWFF3", "[", "s_String", "]"}], ":=", 
  RowBox[{
   RowBox[{"StringCount", "[", 
    RowBox[{"s", ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\"", ",", "\"\<+\>\"", ",", 
       "\"\<-\>\"", ",", "\"\<*\>\"", ",", "\"\</\>\""}], "}"}]}], "]"}], "<=",
    "3"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can now use the name of this function as the second argument to ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Select", "[", 
  RowBox[{"allWFF3", ",", "testWFF3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"(-(-x))\"\>", ",", "\<\"(-x)\"\>", ",", "\<\"x\"\>", 
   ",", "\<\"(x*x)\"\>", ",", "\<\"(x-x)\"\>", ",", "\<\"(x+x)\"\>", 
   ",", "\<\"(x/x)\"\>", ",", "\<\"(x*y)\"\>", ",", "\<\"(x-y)\"\>", 
   ",", "\<\"(x+y)\"\>", ",", "\<\"(x/y)\"\>", ",", "\<\"(x*z)\"\>", 
   ",", "\<\"(x-z)\"\>", ",", "\<\"(x+z)\"\>", ",", "\<\"(x/z)\"\>", 
   ",", "\<\"(-(-y))\"\>", ",", "\<\"(-y)\"\>", ",", "\<\"y\"\>", 
   ",", "\<\"(y*x)\"\>", ",", "\<\"(y-x)\"\>", ",", "\<\"(y+x)\"\>", 
   ",", "\<\"(y/x)\"\>", ",", "\<\"(y*y)\"\>", ",", "\<\"(y-y)\"\>", 
   ",", "\<\"(y+y)\"\>", ",", "\<\"(y/y)\"\>", ",", "\<\"(y*z)\"\>", 
   ",", "\<\"(y-z)\"\>", ",", "\<\"(y+z)\"\>", ",", "\<\"(y/z)\"\>", 
   ",", "\<\"(-(-z))\"\>", ",", "\<\"(-z)\"\>", ",", "\<\"z\"\>", 
   ",", "\<\"(z*x)\"\>", ",", "\<\"(z-x)\"\>", ",", "\<\"(z+x)\"\>", 
   ",", "\<\"(z/x)\"\>", ",", "\<\"(z*y)\"\>", ",", "\<\"(z-y)\"\>", 
   ",", "\<\"(z+y)\"\>", ",", "\<\"(z/y)\"\>", ",", "\<\"(z*z)\"\>", 
   ",", "\<\"(z-z)\"\>", ",", "\<\"(z+z)\"\>", ",", "\<\"(z/z)\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "It is left to the reader to generalize and combine these functions into a \
single function that accepts ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as an argument and returns the list of all well-formed formulae with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " or fewer symbols."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 2", "Subsection",
 CellTags->"SolutionsCE2"],

Cell["\<\
Determine which Fibonacci numbers are divisible by 5, which are divisible by \
7, and which are divisible by 11. Prove that your conjectures are correct. \
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We use the ",
 StyleBox[ButtonBox["Fibonacci",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Fibonacci"], "FunctionLink"],
 " function to determine the Fibonacci numbers. This function applied to an \
integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " returns the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th Fibonacci number."
}], "Text"],

Cell[TextData[{
 "To answer the first part of the question, we want to know which Fibonacci \
numbers are divisible by 5. That is, we want to determine for which ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th Fibonacci divisible by 5. We will construct a list consisting of those \
indices between 1 and 50 for which the corresponding Fibonacci number is \
divisible by 5."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reap", "[", 
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"n", "=", "1"}], ",", 
    RowBox[{"n", "\[LessEqual]", "50"}], ",", 
    RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"Divisible", "[", 
       RowBox[{
        RowBox[{"Fibonacci", "[", "n", "]"}], ",", "5"}], "]"}], ",", 
      RowBox[{"Sow", "[", "n", "]"}]}], "]"}]}], "\[IndentingNewLine]", "]"}],
   "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{
     "5", ",", "10", ",", "15", ",", "20", ",", "25", ",", "30", ",", "35", 
      ",", "40", ",", "45", ",", "50"}], "}"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This list suggests that the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th Fibonacci number is divisible by 5 when ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is. To obtain more evidence, we'll design a function to look for \
counterexamples to the assertion: ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "n"], TraditionalForm]]],
 " is divisible by 5 if and only if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is divisible by 5."
}], "Text"],

Cell[TextData[{
 "We first create a function that accepts a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and checks whether or not ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a counterexample of the assertion that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "n"], TraditionalForm]]],
 " is divisible by 5 if and only if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is divisible by 5. If either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "n"], TraditionalForm]]],
 " is not divisible by 5 when ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is, or ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "n"], TraditionalForm]]],
 " is divisible by 5 when ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is not, then this function will print a message to that effect. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"checkFib5", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "F", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"F", "=", 
      RowBox[{"Fibonacci", "[", "n", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Divisible", "[", 
         RowBox[{"F", ",", "5"}], "]"}], "&&", 
        RowBox[{"Not", "[", 
         RowBox[{"Divisible", "[", 
          RowBox[{"n", ",", "5"}], "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{
         RowBox[{"\"\<Fn=\>\"", "F"}], ",", 
         "\"\< is not divisible by 5, but n=\>\"", ",", "n", ",", 
         "\"\< is\>\""}], "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Divisible", "[", 
         RowBox[{"n", ",", "5"}], "]"}], "&&", 
        RowBox[{"Not", "[", 
         RowBox[{"Divisible", "[", 
          RowBox[{"F", ",", "5"}], "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{
         RowBox[{"\"\<Fn=\>\"", "F"}], ",", 
         "\"\< is divisible by 5, but n=\>\"", ",", "n", ",", 
         "\"\< is not\>\""}], "]"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "In order to check as many Fibonacci numbers as possible, while not spending \
too much time on the process, we will apply the ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " function. The first argument to ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " is an expression to be evaluated. In this case, the expression will be an \
infinite ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop that applies ",
 StyleBox["checkFib5", "Input"],
 ". We make the loop infinite by entering ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " for the test. The second argument to ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " is the time, in seconds, that the expression should be allowed to run. ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " also accepts a third, optional, argument, that is evaluated if the first \
expression fails to finish. If the third argument is not given, the output in \
case the time runs out will be ",
 StyleBox[ButtonBox["$Aborted",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/$Aborted"], "FunctionLink"],
 ". We will use a ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " statement as the third argument to report the largest value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "n"], TraditionalForm]]],
 " was checked. Note that we subtract one from the value of the loop variable \
since we cannot be certain whether the current value was in fact checked or \
if it was aborted."
}], "Text"],

Cell["Below, we allow the test to run for 3 seconds.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeConstrained", "[", 
  RowBox[{
   RowBox[{"For", "[", 
    RowBox[{
     RowBox[{"n", "=", "1"}], ",", "True", ",", 
     RowBox[{"n", "++"}], ",", 
     RowBox[{"checkFib5", "[", "n", "]"}]}], "]"}], ",", "3", ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Print", "[", 
    RowBox[{"\"\<Checked through n=\>\"", ",", 
     RowBox[{"n", "-", "1"}]}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Checked through n=\"\>", "\[InvisibleSpace]", "58275"}],
  SequenceForm["Checked through n=", 58275],
  Editable->False]], "Print"]
}, Open  ]],

Cell["\<\
How many Fibonacci numbers can be checked will depend on your computer. \
Proving the conjecture, as well as forming and proving conjectures for 7 and \
11, is left to the reader.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find and prove formulas for the sum of the first ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th powers of positive integers for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "=", "4"}], ",", "5", ",", "6", ",", "7", ",", "8", ",", "9",
     ",", "10"}], TraditionalForm]]],
 "."
}], "Item1Exercise"],

Cell[TextData[{
 "For what positive integers ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "k"], "-", "n"}], TraditionalForm]]],
 " divisible by ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "?"
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to help you find and prove the formulas sought in Exercises 9, 10, and 11 \
of Section 5.1 of the text. Do not use the symbolic capabilities of the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " command to form your conjectures."
}], "Item1Exercise"],

Cell[TextData[{
 "Find integers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", 
     RowBox[{"n", "+", "1"}]], "+", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "+", "1"}], ")"}], 
     RowBox[{
      RowBox[{"2", "n"}], "-", "1"}]]}], TraditionalForm]]],
 " for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". (Exercises 36 and 37 in Section 5.1 indicate that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "=", "4"}], ",", 
    RowBox[{"d", "=", "21"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "=", "11"}], ",", 
    RowBox[{"d", "=", "133"}]}], TraditionalForm]]],
 " are two such pairs.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Supplementary Exercises 4 and 5 suggest a more general conjecture. Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function to produce evidence for this conjecture."
}], "Item1Exercise"],

Cell[TextData[{
 "Use the ",
 StyleBox["postageBasis", "Input"],
 " algorithm from Section 5.2 of this manual to find the smallest ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that every amount of postage of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " cents or more can be made from stamps worth 78 cents and $5.95."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function that accepts two stamp denominations as input and returns \
the smallest ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that every amount of postage of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " cents or more can be made from the given denominations, or returns ",
 StyleBox[ButtonBox["$Failed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/$Failed"], "FunctionLink"],
 " if it cannot find such an ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Use your function to make a conjecture that describes for which pairs of \
denominations such an ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " exists and for which there is no such ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function to recursively build a set from the following definition: \
basis step: 2 and 3 belong to the set; recursive step: if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are members, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is a member."
}], "Item1Exercise"],

Cell[TextData[{
 "Use the ",
 StyleBox[ButtonBox["Timing",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Timing"], "FunctionLink"],
 " function to compare the performance of ",
 StyleBox["gR", "Input"],
 " and ",
 StyleBox["gF", "Input"],
 " from Section 5.3. Graph the time performance for the two functions. (Be \
sure to ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " ",
 StyleBox["gR", "Input"],
 " prior to every execution so that the comparison is fair.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function that accepts two stamp denominations and returns all \
amounts of postage that can be paid with up to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " stamps."
}], "Item1Exercise"],

Cell[TextData[{
 "The solution provided for Computer Projects 2 is inefficient as a means of \
finding all well-formed formulae involving at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " symbols. This is because formulae which already have more symbols than the \
maximum are retained and used to produce even longer formulae. In particular, \
after ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " iterations, the resulting set includes formulae of up to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     RowBox[{"n", "+", "1"}]], "-", "1"}], TraditionalForm]]],
 " symbols (but not all such formulae). Modify the approach taken in the \
solution to Computer Projects 2 so that strings that include more than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " symbols are pruned at each step of the recursion."
}], "Item1Exercise"],

Cell["\<\
Write a function to compute the number of partitions of a positive integer \
(see Exercise 47 in Section 5.3 of the text).\
\>", "Item1Exercise"],

Cell["\<\
Write a function to compute Ackermann's function (see the prelude to Exercise \
48 in Section 5.3).\
\>", "Item1Exercise"],

Cell[TextData[{
 "Implement Algorithm 3 for computing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"gcd", "(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " from Section 5.4 of the text."
}], "Item1Exercise"],

Cell["\<\
Implement Algorithm 5, the recursive linear search algorithm, from Section \
5.4 of the text. \
\>", "Item1Exercise"],

Cell["\<\
Implement Algorithm 6, the recursive binary search algorithm, from Section \
5.4 of the text. \
\>", "Item1Exercise"],

Cell[TextData[{
 "Compare the performance of your implementations of Algorithm 5 and \
Algorithm 6 as follows: for a variety of values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", let ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " be the list of integers from 1 to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Randomly choose one hundred integers between 1 and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and measure the average of the times taken for each algorithm to find the \
randomly chosen integers. Graph ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " versus the average times. (",
 Cell[BoxData[
  RowBox[{
   RowBox[{"RandomInteger", "[", "{"}], 
   StyleBox["min", "CodeVar"], ",", 
   StyleBox["max", "CodeVar"], "}", ",", 
   StyleBox["count", "CodeVar"], "]"}]], "myCode"],
 " will output a list of ",
 StyleBox["count", "CodeVar"],
 " randomly chosen integers between ",
 StyleBox["min", "CodeVar"],
 " and ",
 StyleBox["max", "CodeVar"],
 ".)"
}], "Item1Exercise"],

Cell["\<\
Create three functions to compute Fibonacci numbers: an iterative function, a \
recursive function that stores values, and a recursive function that does not \
store values. Base your functions on Algorithms 7 and 8 in Section 5.4 of the \
text. Create a graph illustrating the time performance of the three functions.\
\>", "Item1Exercise"],

Cell["\<\
Implement quick sort, described in the prelude to Exercise 50 in Section 5.4 \
of the text. Compare the performance with the merge sort implemented in this \
manual. \
\>", "Item1Exercise"],

Cell["\<\
Implement the algorithm described in Supplementary Exercise 44 for expressing \
a rational number as a sum of Egyptian fractions. \
\>", "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to study the McCarthy 91 function. (See the prelude to problem 45 in the \
Supplementary Exercises of Chapter 5.)"
}], "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{4, Automatic}, {Automatic, 4}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section3a", "Section3b",
                "Section3c", "Section3d", "Section3e", "Section3f", 
               "Section4a", "Section4b", "Section4c", "Section5a", 
               "Section5b", "SolutionsCE2", "SolutionsCP2", "Section1", 
               "Section2", "Section3", "Section4", "Section5", "Solutions", 
               "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section1"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section2"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section3"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section4"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section5"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Solutions"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Exercises"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section2", "Section3", "Section3a", "Section3b", "Section3c", 
               "Section3d", "Section3e", "Section3f", "Section4", "Section4a",
                "Section4b", "Section4c", "Section5", "Section5a", 
               "Section5b", "Solutions", "SolutionsCE2", "SolutionsCP2"}}] 
           SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Exercises"->{
  Cell[290316, 9119, 52, 1, 35, "Section",
   CellTags->"Exercises"]},
 "Section1"->{
  Cell[1451, 52, 68, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[1961, 69, 65, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[36925, 1181, 65, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[49735, 1672, 58, 1, 26, "Subsection",
   CellTags->"Section1c"]},
 "Section2"->{
  Cell[55325, 1898, 80, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section3"->{
  Cell[76743, 2594, 92, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[77033, 2607, 73, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[88107, 3005, 73, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section3c"->{
  Cell[95185, 3256, 66, 1, 26, "Subsection",
   CellTags->"Section3c"]},
 "Section3d"->{
  Cell[108230, 3703, 86, 1, 26, "Subsection",
   CellTags->"Section3d"]},
 "Section3e"->{
  Cell[115349, 3944, 71, 1, 26, "Subsection",
   CellTags->"Section3e"]},
 "Section3f"->{
  Cell[121077, 4167, 62, 1, 26, "Subsection",
   CellTags->"Section3f"]},
 "Section4"->{
  Cell[130686, 4459, 66, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[131230, 4475, 68, 1, 32, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[165309, 5341, 69, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Section4c"->{
  Cell[209320, 6246, 56, 1, 26, "Subsection",
   CellTags->"Section4c"]},
 "Section5"->{
  Cell[226383, 6806, 65, 1, 35, "Section",
   CellTags->"Section5"]},
 "Section5a"->{
  Cell[226810, 6823, 80, 1, 32, "Subsection",
   CellTags->"Section5a"]},
 "Section5b"->{
  Cell[257874, 8062, 84, 1, 26, "Subsection",
   CellTags->"Section5b"]},
 "Solutions"->{
  Cell[264336, 8299, 115, 3, 35, "Section",
   CellTags->"Solutions"]},
 "SolutionsCE2"->{
  Cell[282662, 8881, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE2"]},
 "SolutionsCP2"->{
  Cell[264476, 8306, 68, 1, 32, "Subsection",
   CellTags->"SolutionsCP2"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Exercises", 315651, 9737},
 {"Section1", 315737, 9740},
 {"Section1a", 315819, 9743},
 {"Section1b", 315905, 9746},
 {"Section1c", 315994, 9749},
 {"Section2", 316082, 9752},
 {"Section3", 316166, 9755},
 {"Section3a", 316251, 9758},
 {"Section3b", 316340, 9761},
 {"Section3c", 316429, 9764},
 {"Section3d", 316518, 9767},
 {"Section3e", 316608, 9770},
 {"Section3f", 316698, 9773},
 {"Section4", 316787, 9776},
 {"Section4a", 316873, 9779},
 {"Section4b", 316963, 9782},
 {"Section4c", 317053, 9785},
 {"Section5", 317142, 9788},
 {"Section5a", 317228, 9791},
 {"Section5b", 317318, 9794},
 {"Solutions", 317408, 9797},
 {"SolutionsCE2", 317499, 9800},
 {"SolutionsCP2", 317595, 9803}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 96, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 5}}],
Cell[CellGroupData[{
Cell[700, 27, 31, 0, 55, "Section"],
Cell[734, 29, 680, 18, 65, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1451, 52, 68, 1, 55, "Section",
 CellTags->"Section1"],
Cell[1522, 55, 414, 10, 50, "Text"],
Cell[CellGroupData[{
Cell[1961, 69, 65, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[2029, 72, 98, 2, 20, "Text"],
Cell[2130, 76, 401, 13, 52, "Equation"],
Cell[2534, 91, 160, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[2719, 98, 65, 0, 18, "Subsubsection"],
Cell[2787, 100, 379, 12, 36, "Text"],
Cell[3169, 114, 641, 14, 66, "Text"],
Cell[3813, 130, 1987, 65, 91, "Text"],
Cell[5803, 197, 155, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[5983, 204, 129, 4, 28, "Input"],
Cell[6115, 210, 29, 0, 28, "Output"]
}, Open  ]],
Cell[6159, 213, 766, 21, 51, "Text"],
Cell[CellGroupData[{
Cell[6950, 238, 240, 8, 28, "Input"],
Cell[7193, 248, 678, 11, 59, "Output"]
}, Open  ]],
Cell[7886, 262, 699, 18, 52, "Text"],
Cell[CellGroupData[{
Cell[8610, 284, 297, 10, 28, "Input"],
Cell[8910, 296, 3134, 102, 119, "Output"]
}, Open  ]],
Cell[12059, 401, 321, 13, 20, "Text"],
Cell[12383, 416, 2111, 58, 128, "Text"],
Cell[CellGroupData[{
Cell[14519, 478, 284, 9, 28, "Input"],
Cell[14806, 489, 6477, 111, 228, "Output"]
}, Open  ]],
Cell[21298, 603, 432, 7, 50, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21767, 615, 49, 0, 18, "Subsubsection"],
Cell[21819, 617, 646, 23, 38, "Text"],
Cell[22468, 642, 109, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[22602, 649, 303, 10, 28, "Input",
 InitializationCell->True],
Cell[22908, 661, 167, 4, 28, "Output"]
}, Open  ]],
Cell[23090, 668, 324, 12, 20, "Text"],
Cell[23417, 682, 319, 10, 29, "Equation"],
Cell[23739, 694, 123, 6, 20, "Text"],
Cell[23865, 702, 115, 3, 26, "Equation"],
Cell[23983, 707, 115, 6, 20, "Text"],
Cell[24101, 715, 155, 5, 26, "Equation"],
Cell[24259, 722, 1077, 35, 55, "Text"],
Cell[CellGroupData[{
Cell[25361, 761, 409, 13, 29, "Input",
 InitializationCell->True],
Cell[25773, 776, 1297, 40, 59, "Output"]
}, Open  ]],
Cell[27085, 819, 870, 21, 67, "Text"],
Cell[CellGroupData[{
Cell[27980, 844, 151, 4, 29, "Input"],
Cell[28134, 850, 248, 8, 46, "Output"]
}, Open  ]],
Cell[28397, 861, 1133, 32, 95, "Text"],
Cell[29533, 895, 1869, 65, 123, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31439, 965, 44, 0, 18, "Subsubsection"],
Cell[31486, 967, 141, 3, 20, "Text"],
Cell[31630, 972, 307, 11, 29, "Input",
 InitializationCell->True],
Cell[31940, 985, 214, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[32179, 996, 128, 4, 28, "Input"],
Cell[32310, 1002, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32375, 1007, 57, 1, 28, "Input"],
Cell[32435, 1010, 28, 0, 28, "Output"]
}, Open  ]],
Cell[32478, 1013, 63, 0, 20, "Text"],
Cell[32544, 1015, 1135, 35, 76, "Text"],
Cell[33682, 1052, 454, 17, 20, "Text"],
Cell[CellGroupData[{
Cell[34161, 1073, 127, 4, 28, "Input"],
Cell[34291, 1079, 151, 5, 49, "Output"]
}, Open  ]],
Cell[34457, 1087, 217, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[34699, 1099, 80, 2, 28, "Input"],
Cell[34782, 1103, 216, 8, 46, "Output"]
}, Open  ]],
Cell[35013, 1114, 1109, 33, 67, "Text"],
Cell[CellGroupData[{
Cell[36147, 1151, 284, 9, 28, "Input"],
Cell[36434, 1162, 31, 0, 28, "Output"]
}, Open  ]],
Cell[36480, 1165, 396, 10, 36, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[36925, 1181, 65, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[36993, 1184, 866, 23, 68, "Text"],
Cell[CellGroupData[{
Cell[37884, 1211, 103, 4, 18, "Subsubsection"],
Cell[37990, 1217, 1201, 35, 86, "Text"],
Cell[CellGroupData[{
Cell[39216, 1256, 251, 9, 28, "Input"],
Cell[39470, 1267, 37, 0, 28, "Output"]
}, Open  ]],
Cell[39522, 1270, 274, 7, 36, "Text"],
Cell[39799, 1279, 745, 23, 54, "Text"],
Cell[CellGroupData[{
Cell[40569, 1306, 250, 9, 28, "Input"],
Cell[40822, 1317, 220, 8, 28, "Output"]
}, Open  ]],
Cell[41057, 1328, 206, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[41288, 1338, 61, 1, 29, "Input"],
Cell[41352, 1341, 213, 8, 28, "Output"]
}, Open  ]],
Cell[41580, 1352, 313, 7, 36, "Text"],
Cell[41896, 1361, 301, 10, 21, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42234, 1376, 44, 0, 18, "Subsubsection"],
Cell[42281, 1378, 1096, 37, 68, "Text"],
Cell[43380, 1417, 72, 0, 20, "Text"],
Cell[43455, 1419, 281, 10, 29, "Input",
 InitializationCell->True],
Cell[43739, 1431, 300, 11, 20, "Text"],
Cell[CellGroupData[{
Cell[44064, 1446, 119, 4, 28, "Input"],
Cell[44186, 1452, 28, 0, 28, "Output"]
}, Open  ]],
Cell[44229, 1455, 140, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[44394, 1465, 58, 1, 28, "Input"],
Cell[44455, 1468, 28, 0, 28, "Output"]
}, Open  ]],
Cell[44498, 1471, 43, 0, 20, "Text"],
Cell[44544, 1473, 177, 5, 20, "Text"],
Cell[44724, 1480, 507, 19, 54, "Equation"],
Cell[45234, 1501, 159, 6, 20, "Text"],
Cell[45396, 1509, 875, 32, 54, "Equation"],
Cell[46274, 1543, 158, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[46457, 1552, 304, 11, 28, "Input"],
Cell[46764, 1565, 392, 15, 31, "Output"]
}, Open  ]],
Cell[47171, 1583, 181, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[47377, 1593, 81, 2, 28, "Input"],
Cell[47461, 1597, 143, 5, 28, "Output"]
}, Open  ]],
Cell[47619, 1605, 223, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[47867, 1615, 450, 16, 28, "Input"],
Cell[48320, 1633, 529, 10, 59, "Message"],
Cell[48852, 1645, 31, 0, 28, "Output"]
}, Open  ]],
Cell[48898, 1648, 316, 7, 35, "Text"],
Cell[49217, 1657, 469, 9, 50, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[49735, 1672, 58, 1, 26, "Subsection",
 CellTags->"Section1c"],
Cell[49796, 1675, 975, 31, 72, "Text"],
Cell[50774, 1708, 198, 7, 23, "Text"],
Cell[50975, 1717, 163, 5, 29, "Input",
 InitializationCell->True],
Cell[51141, 1724, 232, 10, 20, "Text"],
Cell[CellGroupData[{
Cell[51398, 1738, 64, 1, 28, "Input"],
Cell[51465, 1741, 28, 0, 28, "Output"]
}, Open  ]],
Cell[51508, 1744, 64, 0, 20, "Text"],
Cell[51575, 1746, 997, 36, 55, "Text"],
Cell[52575, 1784, 469, 19, 23, "Text"],
Cell[CellGroupData[{
Cell[53069, 1807, 149, 4, 28, "Input"],
Cell[53221, 1813, 175, 6, 31, "Output"]
}, Open  ]],
Cell[53411, 1822, 177, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[53613, 1832, 59, 1, 29, "Input"],
Cell[53675, 1835, 357, 12, 30, "Output"]
}, Open  ]],
Cell[54047, 1850, 593, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[54665, 1869, 278, 8, 28, "Input"],
Cell[54946, 1879, 31, 0, 28, "Output"]
}, Open  ]],
Cell[54992, 1882, 284, 10, 23, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[55325, 1898, 80, 1, 35, "Section",
 CellTags->"Section2"],
Cell[55408, 1901, 528, 10, 65, "Text"],
Cell[55939, 1913, 841, 25, 50, "Text"],
Cell[56783, 1940, 888, 25, 65, "Text"],
Cell[CellGroupData[{
Cell[57696, 1969, 39, 0, 18, "Subsubsection"],
Cell[57738, 1971, 843, 31, 50, "Text"],
Cell[58584, 2004, 1763, 66, 74, "Text"],
Cell[60350, 2072, 1001, 34, 52, "Text"],
Cell[CellGroupData[{
Cell[61376, 2110, 946, 26, 119, "Input"],
Cell[62325, 2138, 73, 2, 28, "Output"]
}, Open  ]],
Cell[62413, 2143, 570, 11, 66, "Text"],
Cell[62986, 2156, 1381, 36, 154, "Input",
 InitializationCell->True],
Cell[64370, 2194, 209, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[64604, 2203, 98, 2, 29, "Input"],
Cell[64705, 2207, 73, 2, 28, "Output"]
}, Open  ]],
Cell[64793, 2212, 158, 3, 20, "Text"],
Cell[64954, 2217, 98, 2, 29, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[65089, 2224, 50, 0, 18, "Subsubsection"],
Cell[65142, 2226, 390, 8, 51, "Text"],
Cell[CellGroupData[{
Cell[65557, 2238, 338, 9, 61, "Input"],
Cell[CellGroupData[{
Cell[65920, 2251, 72, 2, 21, "Print"],
Cell[65995, 2255, 72, 2, 21, "Print"],
Cell[66070, 2259, 72, 2, 21, "Print"],
Cell[66145, 2263, 72, 2, 21, "Print"]
}, Open  ]]
}, Open  ]],
Cell[66244, 2269, 909, 26, 65, "Text"],
Cell[67156, 2297, 1034, 36, 35, "Text"],
Cell[68193, 2335, 201, 4, 35, "Text"],
Cell[68397, 2341, 573, 15, 52, "Text"],
Cell[68973, 2358, 209, 6, 45, "Input",
 InitializationCell->True],
Cell[69185, 2366, 1121, 30, 68, "Text"],
Cell[70309, 2398, 2054, 50, 282, "Input",
 InitializationCell->True],
Cell[72366, 2450, 119, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[72510, 2457, 99, 2, 29, "Input"],
Cell[72612, 2461, 421, 14, 28, "Output"]
}, Open  ]],
Cell[73048, 2478, 1173, 35, 85, "Text"],
Cell[74224, 2515, 538, 15, 53, "Text"],
Cell[74765, 2532, 1020, 26, 69, "Text"],
Cell[75788, 2560, 160, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[75973, 2567, 99, 2, 29, "Input"],
Cell[76075, 2571, 302, 7, 24, "Message"],
Cell[76380, 2580, 302, 7, 24, "Message"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[76743, 2594, 92, 1, 35, "Section",
 CellTags->"Section3"],
Cell[76838, 2597, 170, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[77033, 2607, 73, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[77109, 2610, 499, 17, 35, "Text"],
Cell[77611, 2629, 1440, 46, 65, "Text"],
Cell[79054, 2677, 1036, 36, 50, "Text"],
Cell[80093, 2715, 267, 7, 35, "Text"],
Cell[80363, 2724, 209, 7, 29, "Input",
 InitializationCell->True],
Cell[80575, 2733, 228, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[80828, 2744, 104, 3, 28, "Input",
 InitializationCell->True],
Cell[80935, 2749, 28, 0, 28, "Output"]
}, Open  ]],
Cell[80978, 2752, 584, 20, 37, "Text"],
Cell[81565, 2774, 173, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[81763, 2783, 55, 1, 28, "Input"],
Cell[81821, 2786, 31, 0, 28, "Output"]
}, Open  ]],
Cell[81867, 2789, 222, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[82114, 2801, 159, 5, 28, "Input"],
Cell[82276, 2808, 165, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82478, 2817, 36, 0, 18, "Subsubsection"],
Cell[82517, 2819, 626, 15, 66, "Text"],
Cell[CellGroupData[{
Cell[83168, 2838, 63, 1, 28, "Input"],
Cell[83234, 2841, 401, 12, 29, "Output"]
}, Open  ]],
Cell[83650, 2856, 212, 7, 21, "Text"],
Cell[83865, 2865, 1056, 29, 83, "Text"],
Cell[84924, 2896, 472, 16, 37, "Text"],
Cell[85399, 2914, 1649, 58, 81, "Text"],
Cell[87051, 2974, 658, 16, 66, "Text"],
Cell[87712, 2992, 346, 7, 36, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[88107, 3005, 73, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[88183, 3008, 703, 25, 35, "Text"],
Cell[88889, 3035, 112, 3, 20, "Text"],
Cell[89004, 3040, 222, 7, 44, "Input",
 InitializationCell->True],
Cell[89229, 3049, 64, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[89318, 3053, 63, 1, 28, "Input"],
Cell[89384, 3056, 292, 9, 28, "Output"]
}, Open  ]],
Cell[89691, 3068, 89, 2, 20, "Text"],
Cell[89783, 3072, 294, 10, 29, "Input",
 InitializationCell->True],
Cell[90080, 3084, 523, 14, 52, "Text"],
Cell[CellGroupData[{
Cell[90628, 3102, 63, 1, 28, "Input"],
Cell[90694, 3105, 642, 19, 45, "Output"]
}, Open  ]],
Cell[91351, 3127, 1267, 40, 71, "Text"],
Cell[92621, 3169, 129, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[92775, 3177, 170, 5, 28, "Input"],
Cell[92948, 3184, 175, 4, 28, "Output"]
}, Open  ]],
Cell[93138, 3191, 197, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[93360, 3201, 63, 1, 28, "Input"],
Cell[93426, 3204, 1710, 46, 90, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[95185, 3256, 66, 1, 26, "Subsection",
 CellTags->"Section3c"],
Cell[95254, 3259, 1085, 33, 66, "Text"],
Cell[96342, 3294, 254, 6, 21, "Text"],
Cell[96599, 3302, 858, 26, 109, "Input",
 InitializationCell->True],
Cell[97460, 3330, 792, 24, 109, "Input",
 InitializationCell->True],
Cell[98255, 3356, 133, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[98413, 3364, 55, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[98493, 3369, 205, 5, 22, "Print"],
Cell[98701, 3376, 205, 5, 22, "Print"],
Cell[98909, 3383, 205, 5, 22, "Print"],
Cell[99117, 3390, 205, 5, 22, "Print"]
}, Open  ]],
Cell[99337, 3398, 30, 0, 28, "Output"]
}, Open  ]],
Cell[99382, 3401, 1791, 63, 83, "Text"],
Cell[101176, 3466, 1516, 49, 103, "Text"],
Cell[102695, 3517, 102, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[102822, 3525, 55, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[102902, 3530, 205, 5, 22, "Print"],
Cell[103110, 3537, 205, 5, 22, "Print"],
Cell[103318, 3544, 205, 5, 22, "Print"],
Cell[103526, 3551, 205, 5, 22, "Print"],
Cell[103734, 3558, 205, 5, 22, "Print"],
Cell[103942, 3565, 205, 5, 22, "Print"],
Cell[104150, 3572, 205, 5, 22, "Print"]
}, Open  ]],
Cell[104370, 3580, 30, 0, 28, "Output"]
}, Open  ]],
Cell[104415, 3583, 947, 27, 69, "Text"],
Cell[105365, 3612, 631, 20, 53, "Text"],
Cell[105999, 3634, 1082, 32, 87, "Text"],
Cell[107084, 3668, 704, 21, 53, "Text"],
Cell[107791, 3691, 402, 7, 50, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[108230, 3703, 86, 1, 26, "Subsection",
 CellTags->"Section3d"],
Cell[108319, 3706, 652, 23, 38, "Text"],
Cell[108974, 3731, 2060, 59, 47, "Equation"],
Cell[111037, 3792, 139, 3, 20, "Text"],
Cell[111179, 3797, 148, 5, 28, "Input",
 InitializationCell->True],
Cell[111330, 3804, 457, 11, 36, "Text"],
Cell[111790, 3817, 630, 21, 29, "Input",
 InitializationCell->True],
Cell[112423, 3840, 169, 7, 23, "Text"],
Cell[CellGroupData[{
Cell[112617, 3851, 77, 2, 28, "Input"],
Cell[112697, 3855, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112762, 3860, 77, 2, 28, "Input"],
Cell[112842, 3864, 29, 0, 28, "Output"]
}, Open  ]],
Cell[112886, 3867, 438, 15, 24, "Text"],
Cell[CellGroupData[{
Cell[113349, 3886, 307, 10, 28, "Input"],
Cell[113659, 3898, 1264, 25, 200, "Output"]
}, Open  ]],
Cell[114938, 3926, 374, 13, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[115349, 3944, 71, 1, 26, "Subsection",
 CellTags->"Section3e"],
Cell[115423, 3947, 152, 3, 20, "Text"],
Cell[115578, 3952, 132, 5, 20, "Text"],
Cell[115713, 3959, 259, 11, 15, "Text"],
Cell[115975, 3972, 386, 16, 15, "Text"],
Cell[116364, 3990, 118, 3, 20, "Text"],
Cell[116485, 3995, 326, 10, 35, "Text"],
Cell[116814, 4007, 221, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[117060, 4018, 123, 4, 28, "Input",
 InitializationCell->True],
Cell[117186, 4024, 73, 2, 28, "Output"]
}, Open  ]],
Cell[117274, 4029, 879, 29, 53, "Text"],
Cell[118156, 4060, 671, 21, 37, "Text"],
Cell[118830, 4083, 37, 0, 20, "Text"],
Cell[118870, 4085, 704, 21, 76, "Input",
 InitializationCell->True],
Cell[119577, 4108, 92, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[119694, 4116, 111, 3, 28, "Input",
 InitializationCell->True],
Cell[119808, 4121, 105, 2, 28, "Output"]
}, Open  ]],
Cell[119928, 4126, 41, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[119994, 4130, 111, 3, 28, "Input",
 InitializationCell->True],
Cell[120108, 4135, 214, 5, 28, "Output"]
}, Open  ]],
Cell[120337, 4143, 24, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[120386, 4147, 111, 3, 28, "Input",
 InitializationCell->True],
Cell[120500, 4152, 528, 9, 44, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[121077, 4167, 62, 1, 26, "Subsection",
 CellTags->"Section3f"],
Cell[121142, 4170, 183, 4, 35, "Text"],
Cell[121328, 4176, 264, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[121617, 4189, 190, 6, 29, "Input",
 InitializationCell->True],
Cell[121810, 4197, 128, 3, 28, "Output"]
}, Open  ]],
Cell[121953, 4203, 384, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[122362, 4219, 107, 3, 28, "Input",
 InitializationCell->True],
Cell[122472, 4224, 57, 1, 28, "Output"]
}, Open  ]],
Cell[122544, 4228, 274, 6, 36, "Text"],
Cell[CellGroupData[{
Cell[122843, 4238, 58, 1, 28, "Input"],
Cell[122904, 4241, 188, 7, 39, "Output"]
}, Open  ]],
Cell[123107, 4251, 535, 11, 51, "Text"],
Cell[123645, 4264, 251, 8, 21, "Text"],
Cell[123899, 4274, 743, 23, 75, "Input",
 InitializationCell->True],
Cell[124645, 4299, 93, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[124763, 4305, 148, 4, 29, "Input",
 InitializationCell->True],
Cell[124914, 4311, 146, 3, 28, "Output"]
}, Open  ]],
Cell[125075, 4317, 360, 8, 35, "Text"],
Cell[CellGroupData[{
Cell[125460, 4329, 148, 4, 29, "Input",
 InitializationCell->True],
Cell[125611, 4335, 466, 7, 28, "Output"]
}, Open  ]],
Cell[126092, 4345, 75, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[126192, 4349, 148, 4, 29, "Input",
 InitializationCell->True],
Cell[126343, 4355, 1837, 30, 89, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[128217, 4390, 43, 0, 18, "Subsubsection"],
Cell[128263, 4392, 302, 5, 35, "Text"],
Cell[128568, 4399, 750, 22, 107, "Input",
 InitializationCell->True],
Cell[129321, 4423, 292, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[129638, 4432, 156, 5, 29, "Input"],
Cell[129797, 4439, 816, 12, 74, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[130686, 4459, 66, 1, 35, "Section",
 CellTags->"Section4"],
Cell[130755, 4462, 450, 9, 50, "Text"],
Cell[CellGroupData[{
Cell[131230, 4475, 68, 1, 32, "Subsection",
 CellTags->"Section4a"],
Cell[131301, 4478, 828, 29, 41, "Text"],
Cell[132132, 4509, 1099, 36, 64, "Text"],
Cell[CellGroupData[{
Cell[133256, 4549, 35, 0, 18, "Subsubsection"],
Cell[133294, 4551, 914, 34, 59, "Text"],
Cell[134211, 4587, 279, 5, 35, "Text"],
Cell[134493, 4594, 161, 5, 29, "Input",
 InitializationCell->True],
Cell[134657, 4601, 603, 12, 66, "Text"],
Cell[135263, 4615, 727, 21, 52, "Text"],
Cell[135993, 4638, 402, 13, 29, "Input",
 InitializationCell->True],
Cell[136398, 4653, 139, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[136562, 4661, 68, 1, 29, "Input"],
Cell[136633, 4664, 667, 20, 61, "Output"]
}, Open  ]],
Cell[137315, 4687, 642, 12, 81, "Text"],
Cell[137960, 4701, 337, 12, 23, "Text"],
Cell[CellGroupData[{
Cell[138322, 4717, 92, 2, 29, "Input"],
Cell[138417, 4721, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[138482, 4726, 103, 3, 28, "Input"],
Cell[138588, 4731, 28, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[138665, 4737, 35, 0, 18, "Subsubsection"],
Cell[138703, 4739, 1191, 37, 64, "Text"],
Cell[139897, 4778, 119, 3, 20, "Text"],
Cell[140019, 4783, 1262, 40, 93, "Input",
 InitializationCell->True],
Cell[141284, 4825, 200, 9, 23, "Text"],
Cell[CellGroupData[{
Cell[141509, 4838, 92, 2, 29, "Input"],
Cell[141604, 4842, 28, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[141681, 4848, 63, 0, 18, "Subsubsection"],
Cell[141747, 4850, 137, 3, 20, "Text"],
Cell[141887, 4855, 780, 19, 67, "Text"],
Cell[142670, 4876, 1128, 27, 82, "Text"],
Cell[143801, 4905, 590, 18, 45, "Input"],
Cell[144394, 4925, 590, 18, 45, "Input"],
Cell[144987, 4945, 290, 10, 37, "Text"],
Cell[145280, 4957, 572, 13, 52, "Text"],
Cell[CellGroupData[{
Cell[145877, 4974, 263, 8, 29, "Input"],
Cell[146143, 4984, 18643, 337, 232, "Output"]
}, Open  ]],
Cell[164801, 5324, 459, 11, 36, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[165309, 5341, 69, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[165381, 5344, 116, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[165522, 5351, 44, 0, 18, "Subsubsection"],
Cell[165569, 5353, 672, 23, 50, "Text"],
Cell[166244, 5378, 448, 13, 60, "Input",
 InitializationCell->True],
Cell[166695, 5393, 703, 21, 51, "Text"],
Cell[167401, 5416, 176, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[167602, 5425, 64, 1, 28, "Input"],
Cell[167669, 5428, 34, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167740, 5433, 45, 1, 28, "Input"],
Cell[167788, 5436, 34, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[167871, 5442, 44, 0, 18, "Subsubsection"],
Cell[167918, 5444, 532, 14, 52, "Text"],
Cell[168453, 5460, 665, 18, 120, "Input",
 InitializationCell->True],
Cell[169121, 5480, 165, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[169311, 5490, 64, 1, 28, "Input"],
Cell[169378, 5493, 34, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[169461, 5499, 58, 0, 18, "Subsubsection"],
Cell[169522, 5501, 164, 3, 35, "Text"],
Cell[169689, 5506, 329, 9, 37, "Text"],
Cell[170021, 5517, 536, 16, 45, "Input"],
Cell[170560, 5535, 384, 12, 29, "Input"],
Cell[CellGroupData[{
Cell[170969, 5551, 269, 8, 29, "Input"],
Cell[171241, 5561, 35658, 615, 237, "Output"]
}, Open  ]],
Cell[206914, 6179, 403, 8, 50, "Text"],
Cell[207320, 6189, 398, 8, 50, "Text"],
Cell[207721, 6199, 572, 16, 52, "Text"],
Cell[208296, 6217, 700, 16, 66, "Text"],
Cell[208999, 6235, 272, 5, 35, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[209320, 6246, 56, 1, 26, "Subsection",
 CellTags->"Section4c"],
Cell[209379, 6249, 68, 0, 20, "Text"],
Cell[209450, 6251, 527, 12, 53, "Text"],
Cell[209980, 6265, 275, 9, 37, "Text"],
Cell[210258, 6276, 1314, 38, 170, "Input",
 InitializationCell->True],
Cell[211575, 6316, 939, 31, 54, "Text"],
Cell[CellGroupData[{
Cell[212539, 6351, 43, 0, 18, "Subsubsection"],
Cell[212585, 6353, 360, 8, 36, "Text"],
Cell[212948, 6363, 361, 8, 37, "Text"],
Cell[213312, 6373, 1878, 62, 83, "Text"],
Cell[215193, 6437, 1293, 35, 69, "Text"],
Cell[216489, 6474, 1099, 37, 53, "Text"],
Cell[217591, 6513, 96, 4, 21, "Text"],
Cell[217690, 6519, 2710, 75, 275, "Input",
 InitializationCell->True],
Cell[220403, 6596, 100, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[220528, 6604, 163, 4, 29, "Input"],
Cell[220694, 6610, 126, 3, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[220869, 6619, 43, 0, 18, "Subsubsection"],
Cell[220915, 6621, 607, 16, 51, "Text"],
Cell[221525, 6639, 1975, 53, 247, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[223525, 6696, 126, 3, 29, "Input"],
Cell[CellGroupData[{
Cell[223676, 6703, 243, 6, 22, "Print"],
Cell[223922, 6711, 140, 4, 21, "Print"],
Cell[224065, 6717, 168, 5, 21, "Print"],
Cell[224236, 6724, 196, 6, 21, "Print"],
Cell[224435, 6732, 202, 5, 22, "Print"],
Cell[224640, 6739, 42, 0, 22, "Print"],
Cell[224685, 6741, 230, 6, 22, "Print"],
Cell[224918, 6749, 140, 4, 21, "Print"],
Cell[225061, 6755, 168, 5, 21, "Print"],
Cell[225232, 6762, 168, 5, 21, "Print"],
Cell[225403, 6769, 202, 5, 22, "Print"],
Cell[225608, 6776, 42, 0, 22, "Print"],
Cell[225653, 6778, 202, 5, 22, "Print"],
Cell[225858, 6785, 42, 0, 22, "Print"]
}, Open  ]],
Cell[225915, 6788, 83, 2, 28, "Output"]
}, Open  ]],
Cell[226013, 6793, 309, 6, 36, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[226383, 6806, 65, 1, 35, "Section",
 CellTags->"Section5"],
Cell[226451, 6809, 334, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[226810, 6823, 80, 1, 32, "Subsection",
 CellTags->"Section5a"],
Cell[226893, 6826, 455, 17, 36, "Text"],
Cell[227351, 6845, 3320, 86, 420, "Input"],
Cell[230674, 6933, 1446, 52, 66, "Text"],
Cell[232123, 6987, 678, 29, 20, "Text"],
Cell[232804, 7018, 61, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[232890, 7022, 1048, 40, 42, "Item1"],
Cell[233941, 7064, 405, 17, 16, "Item1"],
Cell[234349, 7083, 195, 8, 16, "Item1"],
Cell[234547, 7093, 397, 15, 16, "Item1"],
Cell[234947, 7110, 807, 31, 29, "Item1"],
Cell[235757, 7143, 352, 13, 16, "Item1"]
}, Open  ]],
Cell[236124, 7159, 2513, 105, 67, "Text"],
Cell[238640, 7266, 1545, 60, 35, "Text"],
Cell[240188, 7328, 2408, 96, 65, "Text"],
Cell[242599, 7426, 239, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[242863, 7439, 1096, 43, 29, "Item1"],
Cell[243962, 7484, 406, 17, 16, "Item1"],
Cell[244371, 7503, 1058, 41, 42, "Item1"],
Cell[245432, 7546, 948, 38, 29, "Item1"],
Cell[246383, 7586, 3102, 123, 81, "Item1"]
}, Open  ]],
Cell[249500, 7712, 522, 17, 35, "Text"],
Cell[250025, 7731, 643, 26, 35, "Text"],
Cell[CellGroupData[{
Cell[250693, 7761, 331, 13, 16, "Item1"],
Cell[251027, 7776, 357, 13, 16, "Item1"],
Cell[251387, 7791, 1004, 40, 29, "Item1"],
Cell[252394, 7833, 1115, 42, 31, "Item1"],
Cell[253512, 7877, 494, 21, 16, "Item1"]
}, Open  ]],
Cell[254021, 7901, 281, 12, 20, "Text"],
Cell[254305, 7915, 1033, 40, 35, "Text"],
Cell[255341, 7957, 554, 20, 35, "Text"],
Cell[255898, 7979, 1939, 78, 51, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[257874, 8062, 84, 1, 26, "Subsection",
 CellTags->"Section5b"],
Cell[257961, 8065, 130, 4, 21, "Text"],
Cell[258094, 8071, 1285, 36, 170, "Input"],
Cell[259382, 8109, 767, 27, 51, "Text"],
Cell[260152, 8138, 757, 26, 51, "Text"],
Cell[260912, 8166, 620, 21, 36, "Text"],
Cell[261535, 8189, 579, 22, 32, "Text"],
Cell[262117, 8213, 643, 24, 35, "Text"],
Cell[262763, 8239, 1159, 39, 69, "Text"],
Cell[263925, 8280, 362, 13, 21, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[264336, 8299, 115, 3, 35, "Section",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[264476, 8306, 68, 1, 32, "Subsection",
 CellTags->"SolutionsCP2"],
Cell[264547, 8309, 613, 25, 55, "Problem"],
Cell[265163, 8336, 271, 8, 35, "Text"],
Cell[265437, 8346, 120, 3, 20, "Text"],
Cell[265560, 8351, 276, 12, 15, "Text"],
Cell[265839, 8365, 778, 34, 15, "Text"],
Cell[266620, 8401, 214, 4, 35, "Text"],
Cell[266837, 8407, 811, 28, 50, "Text"],
Cell[267651, 8437, 590, 15, 50, "Text"],
Cell[CellGroupData[{
Cell[268266, 8456, 44, 0, 18, "Subsubsection"],
Cell[268313, 8458, 260, 7, 37, "Text"],
Cell[268576, 8467, 688, 15, 68, "Text"],
Cell[269267, 8484, 1364, 40, 108, "Input",
 InitializationCell->True],
Cell[270634, 8526, 250, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[270909, 8539, 146, 4, 29, "Input"],
Cell[271058, 8545, 1017, 16, 77, "Output"]
}, Open  ]],
Cell[272090, 8564, 263, 7, 21, "Text"],
Cell[272356, 8573, 533, 15, 52, "Text"],
Cell[272892, 8590, 773, 21, 121, "Input",
 InitializationCell->True],
Cell[273668, 8613, 862, 22, 83, "Text"],
Cell[274533, 8637, 155, 5, 21, "Text"],
Cell[274691, 8644, 108, 3, 28, "Input"],
Cell[274802, 8649, 75, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[274902, 8653, 65, 1, 29, "Input"],
Cell[274970, 8656, 31, 0, 28, "Output"]
}, Open  ]],
Cell[275016, 8659, 57, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[275098, 8663, 215, 6, 28, "Input"],
Cell[275316, 8671, 795, 13, 77, "Output"]
}, Open  ]],
Cell[276126, 8687, 284, 5, 35, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[276447, 8697, 40, 0, 18, "Subsubsection"],
Cell[276490, 8699, 485, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[277000, 8716, 77, 1, 29, "Input"],
Cell[277080, 8719, 28, 0, 28, "Output"]
}, Open  ]],
Cell[277123, 8722, 727, 14, 66, "Text"],
Cell[CellGroupData[{
Cell[277875, 8740, 171, 4, 29, "Input"],
Cell[278049, 8746, 28, 0, 28, "Output"]
}, Open  ]],
Cell[278092, 8749, 607, 17, 36, "Text"],
Cell[CellGroupData[{
Cell[278724, 8770, 276, 7, 29, "Input"],
Cell[279003, 8779, 28, 0, 28, "Output"]
}, Open  ]],
Cell[279046, 8782, 1072, 25, 83, "Text"],
Cell[280121, 8809, 328, 8, 36, "Text"],
Cell[280452, 8819, 385, 11, 29, "Input",
 InitializationCell->True],
Cell[280840, 8832, 208, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[281073, 8842, 95, 2, 28, "Input"],
Cell[281171, 8846, 1093, 17, 77, "Output"]
}, Open  ]],
Cell[282279, 8866, 334, 9, 35, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[282662, 8881, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE2"],
Cell[282745, 8884, 181, 3, 55, "Problem"],
Cell[282929, 8889, 432, 15, 36, "Text"],
Cell[283364, 8906, 459, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[283848, 8921, 464, 13, 60, "Input"],
Cell[284315, 8936, 253, 7, 28, "Output"]
}, Open  ]],
Cell[284583, 8946, 500, 16, 35, "Text"],
Cell[285086, 8964, 858, 29, 50, "Text"],
Cell[285947, 8995, 1419, 37, 154, "Input",
 InitializationCell->True],
Cell[287369, 9034, 2004, 49, 129, "Text"],
Cell[289376, 9085, 62, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[289463, 9089, 403, 11, 45, "Input"],
Cell[289869, 9102, 180, 4, 22, "Print"]
}, Open  ]],
Cell[290064, 9109, 203, 4, 35, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[290316, 9119, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[290393, 9124, 458, 17, 33, "Item1Exercise"],
Cell[290854, 9143, 383, 16, 23, "Item1Exercise"],
Cell[291240, 9161, 384, 10, 34, "Item1Exercise"],
Cell[291627, 9173, 924, 37, 36, "Item1Exercise"],
Cell[292554, 9212, 321, 9, 34, "Item1Exercise"],
Cell[292878, 9223, 365, 10, 34, "Item1Exercise"],
Cell[293246, 9235, 788, 23, 47, "Item1Exercise"],
Cell[294037, 9260, 412, 13, 33, "Item1Exercise"],
Cell[294452, 9275, 544, 17, 35, "Item1Exercise"],
Cell[294999, 9294, 222, 6, 20, "Item1Exercise"],
Cell[295224, 9302, 880, 21, 75, "Item1Exercise"],
Cell[296107, 9325, 155, 3, 20, "Item1Exercise"],
Cell[296265, 9330, 132, 3, 20, "Item1Exercise"],
Cell[296400, 9335, 217, 7, 20, "Item1Exercise"],
Cell[296620, 9344, 127, 3, 20, "Item1Exercise"],
Cell[296750, 9349, 127, 3, 20, "Item1Exercise"],
Cell[296880, 9354, 1037, 32, 61, "Item1Exercise"],
Cell[297920, 9388, 351, 5, 46, "Item1Exercise"],
Cell[298274, 9395, 199, 4, 33, "Item1Exercise"],
Cell[298476, 9401, 163, 3, 33, "Item1Exercise"],
Cell[298642, 9406, 212, 6, 33, "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
